# Copyright 2020 The gRPC Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Testing the done callbacks mechanism."""

import asyncio
import logging
import unittest

import grpc
from grpc.experimental import aio

from src.proto.grpc.testing import messages_pb2
from src.proto.grpc.testing import test_pb2_grpc
from tests_aio.unit._common import inject_callbacks
from tests_aio.unit._test_base import AioTestBase
from tests_aio.unit._test_server import start_test_server

_NUM_STREAM_RESPONSES = 5
_REQUEST_PAYLOAD_SIZE = 7
_RESPONSE_PAYLOAD_SIZE = 42
_REQUEST = b"\x01\x02\x03"
_RESPONSE = b"\x04\x05\x06"
_TEST_METHOD = "/test/Test"
_FAKE_METHOD = "/test/Fake"


class TestClientSideDoneCallback(AioTestBase):
    async def setUp(self):
        address, self._server = await start_test_server()
        self._channel = aio.insecure_channel(address)
        self._stub = test_pb2_grpc.TestServiceStub(self._channel)

    async def tearDown(self):
        await self._channel.close()
        await self._server.stop(None)

    async def test_add_after_done(self):
        call = self._stub.UnaryCall(messages_pb2.SimpleRequest())
        self.assertEqual(grpc.StatusCode.OK, await call.code())

        validation = inject_callbacks(call)
        await validation

    async def test_unary_unary(self):
        call = self._stub.UnaryCall(messages_pb2.SimpleRequest())
        validation = inject_callbacks(call)

        self.assertEqual(grpc.StatusCode.OK, await call.code())

        await validation

    async def test_unary_stream(self):
        request = messages_pb2.StreamingOutputCallRequest()
        for _ in range(_NUM_STREAM_RESPONSES):
            request.response_parameters.append(
                messages_pb2.ResponseParameters(size=_RESPONSE_PAYLOAD_SIZE)
            )

        call = self._stub.StreamingOutputCall(request)
        validation = inject_callbacks(call)

        response_cnt = 0
        async for response in call:
            response_cnt += 1
            self.assertIsInstance(
                response, messages_pb2.StreamingOutputCallResponse
            )
            self.assertEqual(_RESPONSE_PAYLOAD_SIZE, len(response.payload.body))

        self.assertEqual(_NUM_STREAM_RESPONSES, response_cnt)
        self.assertEqual(grpc.StatusCode.OK, await call.code())

        await validation

    async def test_stream_unary(self):
        payload = messages_pb2.Payload(body=b"\0" * _REQUEST_PAYLOAD_SIZE)
        request = messages_pb2.StreamingInputCallRequest(payload=payload)

        async def gen():
            for _ in range(_NUM_STREAM_RESPONSES):
                yield request

        call = self._stub.StreamingInputCall(gen())
        validation = inject_callbacks(call)

        response = await call
        self.assertIsInstance(response, messages_pb2.StreamingInputCallResponse)
        self.assertEqual(
            _NUM_STREAM_RESPONSES * _REQUEST_PAYLOAD_SIZE,
            response.aggregated_payload_size,
        )
        self.assertEqual(grpc.StatusCode.OK, await call.code())

        await validation

    async def test_stream_stream(self):
        call = self._stub.FullDuplexCall()
        validation = inject_callbacks(call)

        request = messages_pb2.StreamingOutputCallRequest()
        request.response_parameters.append(
            messages_pb2.ResponseParameters(size=_RESPONSE_PAYLOAD_SIZE)
        )

        for _ in range(_NUM_STREAM_RESPONSES):
            await call.write(request)
            response = await call.read()
            self.assertIsInstance(
                response, messages_pb2.StreamingOutputCallResponse
            )
            self.assertEqual(_RESPONSE_PAYLOAD_SIZE, len(response.payload.body))

        await call.done_writing()

        self.assertEqual(grpc.StatusCode.OK, await call.code())
        await validation


class TestServerSideDoneCallback(AioTestBase):
    async def setUp(self):
        self._server = aio.server()
        port = self._server.add_insecure_port("[::]:0")
        self._channel = aio.insecure_channel("localhost:%d" % port)

    async def tearDown(self):
        await self._channel.close()
        await self._server.stop(None)

    async def _register_method_handler(self, method_handler):
        """Registers method handler and starts the server"""
        generic_handler = grpc.method_handlers_generic_handler(
            "test",
            dict(Test=method_handler),
        )
        self._server.add_generic_rpc_handlers((generic_handler,))
        await self._server.start()

    async def test_unary_unary(self):
        validation_future = self.loop.create_future()

        async def test_handler(request: bytes, context: aio.ServicerContext):
            self.assertEqual(_REQUEST, request)
            validation_future.set_result(inject_callbacks(context))
            return _RESPONSE

        await self._register_method_handler(
            grpc.unary_unary_rpc_method_handler(test_handler)
        )
        response = await self._channel.unary_unary(_TEST_METHOD)(_REQUEST)
        self.assertEqual(_RESPONSE, response)

        validation = await validation_future
        await validation

    async def test_unary_stream(self):
        validation_future = self.loop.create_future()

        async def test_handler(request: bytes, context: aio.ServicerContext):
            self.assertEqual(_REQUEST, request)
            validation_future.set_result(inject_callbacks(context))
            for _ in range(_NUM_STREAM_RESPONSES):
                yield _RESPONSE

        await self._register_method_handler(
            grpc.unary_stream_rpc_method_handler(test_handler)
        )
        call = self._channel.unary_stream(_TEST_METHOD)(_REQUEST)
        async for response in call:
            self.assertEqual(_RESPONSE, response)

        validation = await validation_future
        await validation

    async def test_stream_unary(self):
        validation_future = self.loop.create_future()

        async def test_handler(request_iterator, context: aio.ServicerContext):
            validation_future.set_result(inject_callbacks(context))

            async for request in request_iterator:
                self.assertEqual(_REQUEST, request)
            return _RESPONSE

        await self._register_method_handler(
            grpc.stream_unary_rpc_method_handler(test_handler)
        )
        call = self._channel.stream_unary(_TEST_METHOD)()
        for _ in range(_NUM_STREAM_RESPONSES):
            await call.write(_REQUEST)
        await call.done_writing()
        self.assertEqual(_RESPONSE, await call)

        validation = await validation_future
        await validation

    async def test_stream_stream(self):
        validation_future = self.loop.create_future()

        async def test_handler(request_iterator, context: aio.ServicerContext):
            validation_future.set_result(inject_callbacks(context))

            async for request in request_iterator:
                self.assertEqual(_REQUEST, request)
            return _RESPONSE

        await self._register_method_handler(
            grpc.stream_stream_rpc_method_handler(test_handler)
        )
        call = self._channel.stream_stream(_TEST_METHOD)()
        for _ in range(_NUM_STREAM_RESPONSES):
            await call.write(_REQUEST)
        await call.done_writing()
        async for response in call:
            self.assertEqual(_RESPONSE, response)

        validation = await validation_future
        await validation

    async def test_error_in_handler(self):
        """Errors in the handler still triggers callbacks."""
        validation_future = self.loop.create_future()

        async def test_handler(request: bytes, context: aio.ServicerContext):
            self.assertEqual(_REQUEST, request)
            validation_future.set_result(inject_callbacks(context))
            raise RuntimeError("A test RuntimeError")

        await self._register_method_handler(
            grpc.unary_unary_rpc_method_handler(test_handler)
        )
        with self.assertRaises(aio.AioRpcError) as exception_context:
            await self._channel.unary_unary(_TEST_METHOD)(_REQUEST)
        rpc_error = exception_context.exception
        self.assertEqual(grpc.StatusCode.UNKNOWN, rpc_error.code())

        validation = await validation_future
        await validation

    async def test_error_in_callback(self):
        """Errors in the callback won't be propagated to client."""
        validation_future = self.loop.create_future()

        async def test_handler(request: bytes, context: aio.ServicerContext):
            self.assertEqual(_REQUEST, request)

            def exception_raiser(unused_context):
                raise RuntimeError("A test RuntimeError")

            context.add_done_callback(exception_raiser)
            validation_future.set_result(inject_callbacks(context))
            return _RESPONSE

        await self._register_method_handler(
            grpc.unary_unary_rpc_method_handler(test_handler)
        )

        response = await self._channel.unary_unary(_TEST_METHOD)(_REQUEST)
        self.assertEqual(_RESPONSE, response)

        # Following callbacks won't be invoked, if one of the callback crashed.
        validation = await validation_future
        with self.assertRaises(asyncio.TimeoutError):
            await validation

        # Invoke RPC one more time to ensure the toxic callback won't break the
        # server.
        with self.assertRaises(aio.AioRpcError) as exception_context:
            await self._channel.unary_unary(_FAKE_METHOD)(_REQUEST)
        rpc_error = exception_context.exception
        self.assertEqual(grpc.StatusCode.UNIMPLEMENTED, rpc_error.code())


if __name__ == "__main__":
    logging.basicConfig(level=logging.DEBUG)
    unittest.main(verbosity=2)



# Copyright 2015 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Insecure client-server interoperability as a unit test."""

import unittest

import grpc

from src.proto.grpc.testing import test_pb2_grpc
from tests.interop import _intraop_test_case
from tests.interop import server
from tests.interop import service
from tests.unit import test_common


class InsecureIntraopTest(
    _intraop_test_case.IntraopTestCase, unittest.TestCase
):
    def setUp(self):
        self.server = test_common.test_server()
        test_pb2_grpc.add_TestServiceServicer_to_server(
            service.TestService(), self.server
        )
        port = self.server.add_insecure_port("[::]:0")
        self.server.start()
        self.stub = test_pb2_grpc.TestServiceStub(
            grpc.insecure_channel("localhost:{}".format(port))
        )

    def tearDown(self):
        self.server.stop(None)


if __name__ == "__main__":
    unittest.main(verbosity=2)



# Copyright 2016 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import logging
import unittest

import grpc

from tests.unit import test_common
from tests.unit.framework.common import test_constants

_REQUEST = b""
_RESPONSE = b""

_SERVICE_NAME = "test"
_UNARY_UNARY = "UnaryUnary"
_UNARY_STREAM = "UnaryStream"
_STREAM_UNARY = "StreamUnary"
_STREAM_STREAM = "StreamStream"


def handle_unary_unary(request, servicer_context):
    return _RESPONSE


def handle_unary_stream(request, servicer_context):
    for _ in range(test_constants.STREAM_LENGTH):
        yield _RESPONSE


def handle_stream_unary(request_iterator, servicer_context):
    for request in request_iterator:
        pass
    return _RESPONSE


def handle_stream_stream(request_iterator, servicer_context):
    for request in request_iterator:
        yield _RESPONSE


class _MethodHandler(grpc.RpcMethodHandler):
    def __init__(self, request_streaming, response_streaming):
        self.request_streaming = request_streaming
        self.response_streaming = response_streaming
        self.request_deserializer = None
        self.response_serializer = None
        self.unary_unary = None
        self.unary_stream = None
        self.stream_unary = None
        self.stream_stream = None
        if self.request_streaming and self.response_streaming:
            self.stream_stream = handle_stream_stream
        elif self.request_streaming:
            self.stream_unary = handle_stream_unary
        elif self.response_streaming:
            self.unary_stream = handle_unary_stream
        else:
            self.unary_unary = handle_unary_unary


_METHOD_HANDLERS = {
    _UNARY_UNARY: _MethodHandler(False, False),
    _UNARY_STREAM: _MethodHandler(False, True),
    _STREAM_UNARY: _MethodHandler(True, False),
    _STREAM_STREAM: _MethodHandler(True, True),
}


class EmptyMessageTest(unittest.TestCase):
    def setUp(self):
        self._server = test_common.test_server()
        self._server.add_registered_method_handlers(
            _SERVICE_NAME, _METHOD_HANDLERS
        )
        port = self._server.add_insecure_port("[::]:0")
        self._server.start()
        self._channel = grpc.insecure_channel("localhost:%d" % port)

    def tearDown(self):
        self._server.stop(0)
        self._channel.close()

    def testUnaryUnary(self):
        response = self._channel.unary_unary(
            grpc._common.fully_qualified_method(_SERVICE_NAME, _UNARY_UNARY),
            _registered_method=True,
        )(_REQUEST)
        self.assertEqual(_RESPONSE, response)

    def testUnaryStream(self):
        response_iterator = self._channel.unary_stream(
            grpc._common.fully_qualified_method(_SERVICE_NAME, _UNARY_STREAM),
            _registered_method=True,
        )(_REQUEST)
        self.assertSequenceEqual(
            [_RESPONSE] * test_constants.STREAM_LENGTH, list(response_iterator)
        )

    def testStreamUnary(self):
        response = self._channel.stream_unary(
            grpc._common.fully_qualified_method(_SERVICE_NAME, _STREAM_UNARY),
            _registered_method=True,
        )(iter([_REQUEST] * test_constants.STREAM_LENGTH))
        self.assertEqual(_RESPONSE, response)

    def testStreamStream(self):
        response_iterator = self._channel.stream_stream(
            grpc._common.fully_qualified_method(_SERVICE_NAME, _STREAM_STREAM),
            _registered_method=True,
        )(iter([_REQUEST] * test_constants.STREAM_LENGTH))
        self.assertSequenceEqual(
            [_RESPONSE] * test_constants.STREAM_LENGTH, list(response_iterator)
        )


if __name__ == "__main__":
    logging.basicConfig()
    unittest.main(verbosity=2)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: envoy/service/discovery/v3/discovery.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'envoy/service/discovery/v3/discovery.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from envoy.config.core.v3 import base_pb2 as envoy_dot_config_dot_core_dot_v3_dot_base__pb2
from google.protobuf import any_pb2 as google_dot_protobuf_dot_any__pb2
from google.protobuf import duration_pb2 as google_dot_protobuf_dot_duration__pb2
from google.rpc import status_pb2 as google_dot_rpc_dot_status__pb2
from udpa.annotations import status_pb2 as udpa_dot_annotations_dot_status__pb2
from udpa.annotations import versioning_pb2 as udpa_dot_annotations_dot_versioning__pb2
from validate import validate_pb2 as validate_dot_validate__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n*envoy/service/discovery/v3/discovery.proto\x12\x1a\x65nvoy.service.discovery.v3\x1a\x1f\x65nvoy/config/core/v3/base.proto\x1a\x19google/protobuf/any.proto\x1a\x1egoogle/protobuf/duration.proto\x1a\x17google/rpc/status.proto\x1a\x1dudpa/annotations/status.proto\x1a!udpa/annotations/versioning.proto\x1a\x17validate/validate.proto\"\xb9\x01\n\x0fResourceLocator\x12\x0c\n\x04name\x18\x01 \x01(\t\x12^\n\x12\x64ynamic_parameters\x18\x02 \x03(\x0b\x32\x42.envoy.service.discovery.v3.ResourceLocator.DynamicParametersEntry\x1a\x38\n\x16\x44ynamicParametersEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"|\n\x0cResourceName\x12\x0c\n\x04name\x18\x01 \x01(\t\x12^\n\x1d\x64ynamic_parameter_constraints\x18\x02 \x01(\x0b\x32\x37.envoy.service.discovery.v3.DynamicParameterConstraints\"z\n\rResourceError\x12?\n\rresource_name\x18\x01 \x01(\x0b\x32(.envoy.service.discovery.v3.ResourceName\x12(\n\x0c\x65rror_detail\x18\x02 \x01(\x0b\x32\x12.google.rpc.Status\"\xac\x02\n\x10\x44iscoveryRequest\x12\x14\n\x0cversion_info\x18\x01 \x01(\t\x12(\n\x04node\x18\x02 \x01(\x0b\x32\x1a.envoy.config.core.v3.Node\x12\x16\n\x0eresource_names\x18\x03 \x03(\t\x12\x46\n\x11resource_locators\x18\x07 \x03(\x0b\x32+.envoy.service.discovery.v3.ResourceLocator\x12\x10\n\x08type_url\x18\x04 \x01(\t\x12\x16\n\x0eresponse_nonce\x18\x05 \x01(\t\x12(\n\x0c\x65rror_detail\x18\x06 \x01(\x0b\x32\x12.google.rpc.Status:$\x9a\xc5\x88\x1e\x1f\n\x1d\x65nvoy.api.v2.DiscoveryRequest\"\xa9\x02\n\x11\x44iscoveryResponse\x12\x14\n\x0cversion_info\x18\x01 \x01(\t\x12\'\n\tresources\x18\x02 \x03(\x0b\x32\x14.google.protobuf.Any\x12\x0e\n\x06\x63\x61nary\x18\x03 \x01(\x08\x12\x10\n\x08type_url\x18\x04 \x01(\t\x12\r\n\x05nonce\x18\x05 \x01(\t\x12\x39\n\rcontrol_plane\x18\x06 \x01(\x0b\x32\".envoy.config.core.v3.ControlPlane\x12\x42\n\x0fresource_errors\x18\x07 \x03(\x0b\x32).envoy.service.discovery.v3.ResourceError:%\x9a\xc5\x88\x1e \n\x1e\x65nvoy.api.v2.DiscoveryResponse\"\xdf\x04\n\x15\x44\x65ltaDiscoveryRequest\x12(\n\x04node\x18\x01 \x01(\x0b\x32\x1a.envoy.config.core.v3.Node\x12\x10\n\x08type_url\x18\x02 \x01(\t\x12 \n\x18resource_names_subscribe\x18\x03 \x03(\t\x12\"\n\x1aresource_names_unsubscribe\x18\x04 \x03(\t\x12P\n\x1bresource_locators_subscribe\x18\x08 \x03(\x0b\x32+.envoy.service.discovery.v3.ResourceLocator\x12R\n\x1dresource_locators_unsubscribe\x18\t \x03(\x0b\x32+.envoy.service.discovery.v3.ResourceLocator\x12q\n\x19initial_resource_versions\x18\x05 \x03(\x0b\x32N.envoy.service.discovery.v3.DeltaDiscoveryRequest.InitialResourceVersionsEntry\x12\x16\n\x0eresponse_nonce\x18\x06 \x01(\t\x12(\n\x0c\x65rror_detail\x18\x07 \x01(\x0b\x32\x12.google.rpc.Status\x1a>\n\x1cInitialResourceVersionsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01:)\x9a\xc5\x88\x1e$\n\"envoy.api.v2.DeltaDiscoveryRequest\"\x9f\x03\n\x16\x44\x65ltaDiscoveryResponse\x12\x1b\n\x13system_version_info\x18\x01 \x01(\t\x12\x37\n\tresources\x18\x02 \x03(\x0b\x32$.envoy.service.discovery.v3.Resource\x12\x10\n\x08type_url\x18\x04 \x01(\t\x12\x19\n\x11removed_resources\x18\x06 \x03(\t\x12H\n\x16removed_resource_names\x18\x08 \x03(\x0b\x32(.envoy.service.discovery.v3.ResourceName\x12\r\n\x05nonce\x18\x05 \x01(\t\x12\x39\n\rcontrol_plane\x18\x07 \x01(\x0b\x32\".envoy.config.core.v3.ControlPlane\x12\x42\n\x0fresource_errors\x18\t \x03(\x0b\x32).envoy.service.discovery.v3.ResourceError:*\x9a\xc5\x88\x1e%\n#envoy.api.v2.DeltaDiscoveryResponse\"\xb6\x05\n\x1b\x44ynamicParameterConstraints\x12^\n\nconstraint\x18\x01 \x01(\x0b\x32H.envoy.service.discovery.v3.DynamicParameterConstraints.SingleConstraintH\x00\x12`\n\x0eor_constraints\x18\x02 \x01(\x0b\x32\x46.envoy.service.discovery.v3.DynamicParameterConstraints.ConstraintListH\x00\x12\x61\n\x0f\x61nd_constraints\x18\x03 \x01(\x0b\x32\x46.envoy.service.discovery.v3.DynamicParameterConstraints.ConstraintListH\x00\x12R\n\x0fnot_constraints\x18\x04 \x01(\x0b\x32\x37.envoy.service.discovery.v3.DynamicParameterConstraintsH\x00\x1a\xb5\x01\n\x10SingleConstraint\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\x0f\n\x05value\x18\x02 \x01(\tH\x00\x12\x61\n\x06\x65xists\x18\x03 \x01(\x0b\x32O.envoy.service.discovery.v3.DynamicParameterConstraints.SingleConstraint.ExistsH\x00\x1a\x08\n\x06\x45xistsB\x16\n\x0f\x63onstraint_type\x12\x03\xf8\x42\x01\x1a^\n\x0e\x43onstraintList\x12L\n\x0b\x63onstraints\x18\x01 \x03(\x0b\x32\x37.envoy.service.discovery.v3.DynamicParameterConstraintsB\x06\n\x04type\"\x8b\x03\n\x08Resource\x12\x0c\n\x04name\x18\x03 \x01(\t\x12?\n\rresource_name\x18\x08 \x01(\x0b\x32(.envoy.service.discovery.v3.ResourceName\x12\x0f\n\x07\x61liases\x18\x04 \x03(\t\x12\x0f\n\x07version\x18\x01 \x01(\t\x12&\n\x08resource\x18\x02 \x01(\x0b\x32\x14.google.protobuf.Any\x12&\n\x03ttl\x18\x06 \x01(\x0b\x32\x19.google.protobuf.Duration\x12H\n\rcache_control\x18\x07 \x01(\x0b\x32\x31.envoy.service.discovery.v3.Resource.CacheControl\x12\x30\n\x08metadata\x18\t \x01(\x0b\x32\x1e.envoy.config.core.v3.Metadata\x1a$\n\x0c\x43\x61\x63heControl\x12\x14\n\x0c\x64o_not_cache\x18\x01 \x01(\x08:\x1c\x9a\xc5\x88\x1e\x17\n\x15\x65nvoy.api.v2.ResourceB\x93\x01\n(io.envoyproxy.envoy.service.discovery.v3B\x0e\x44iscoveryProtoP\x01ZMgithub.com/envoyproxy/go-control-plane/envoy/service/discovery/v3;discoveryv3\xba\x80\xc8\xd1\x06\x02\x10\x02\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'envoy.service.discovery.v3.discovery_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n(io.envoyproxy.envoy.service.discovery.v3B\016DiscoveryProtoP\001ZMgithub.com/envoyproxy/go-control-plane/envoy/service/discovery/v3;discoveryv3\272\200\310\321\006\002\020\002'
  _globals['_RESOURCELOCATOR_DYNAMICPARAMETERSENTRY']._loaded_options = None
  _globals['_RESOURCELOCATOR_DYNAMICPARAMETERSENTRY']._serialized_options = b'8\001'
  _globals['_DISCOVERYREQUEST']._loaded_options = None
  _globals['_DISCOVERYREQUEST']._serialized_options = b'\232\305\210\036\037\n\035envoy.api.v2.DiscoveryRequest'
  _globals['_DISCOVERYRESPONSE']._loaded_options = None
  _globals['_DISCOVERYRESPONSE']._serialized_options = b'\232\305\210\036 \n\036envoy.api.v2.DiscoveryResponse'
  _globals['_DELTADISCOVERYREQUEST_INITIALRESOURCEVERSIONSENTRY']._loaded_options = None
  _globals['_DELTADISCOVERYREQUEST_INITIALRESOURCEVERSIONSENTRY']._serialized_options = b'8\001'
  _globals['_DELTADISCOVERYREQUEST']._loaded_options = None
  _globals['_DELTADISCOVERYREQUEST']._serialized_options = b'\232\305\210\036$\n\"envoy.api.v2.DeltaDiscoveryRequest'
  _globals['_DELTADISCOVERYRESPONSE']._loaded_options = None
  _globals['_DELTADISCOVERYRESPONSE']._serialized_options = b'\232\305\210\036%\n#envoy.api.v2.DeltaDiscoveryResponse'
  _globals['_DYNAMICPARAMETERCONSTRAINTS_SINGLECONSTRAINT'].oneofs_by_name['constraint_type']._loaded_options = None
  _globals['_DYNAMICPARAMETERCONSTRAINTS_SINGLECONSTRAINT'].oneofs_by_name['constraint_type']._serialized_options = b'\370B\001'
  _globals['_RESOURCE']._loaded_options = None
  _globals['_RESOURCE']._serialized_options = b'\232\305\210\036\027\n\025envoy.api.v2.Resource'
  _globals['_RESOURCELOCATOR']._serialized_start=283
  _globals['_RESOURCELOCATOR']._serialized_end=468
  _globals['_RESOURCELOCATOR_DYNAMICPARAMETERSENTRY']._serialized_start=412
  _globals['_RESOURCELOCATOR_DYNAMICPARAMETERSENTRY']._serialized_end=468
  _globals['_RESOURCENAME']._serialized_start=470
  _globals['_RESOURCENAME']._serialized_end=594
  _globals['_RESOURCEERROR']._serialized_start=596
  _globals['_RESOURCEERROR']._serialized_end=718
  _globals['_DISCOVERYREQUEST']._serialized_start=721
  _globals['_DISCOVERYREQUEST']._serialized_end=1021
  _globals['_DISCOVERYRESPONSE']._serialized_start=1024
  _globals['_DISCOVERYRESPONSE']._serialized_end=1321
  _globals['_DELTADISCOVERYREQUEST']._serialized_start=1324
  _globals['_DELTADISCOVERYREQUEST']._serialized_end=1931
  _globals['_DELTADISCOVERYREQUEST_INITIALRESOURCEVERSIONSENTRY']._serialized_start=1826
  _globals['_DELTADISCOVERYREQUEST_INITIALRESOURCEVERSIONSENTRY']._serialized_end=1888
  _globals['_DELTADISCOVERYRESPONSE']._serialized_start=1934
  _globals['_DELTADISCOVERYRESPONSE']._serialized_end=2349
  _globals['_DYNAMICPARAMETERCONSTRAINTS']._serialized_start=2352
  _globals['_DYNAMICPARAMETERCONSTRAINTS']._serialized_end=3046
  _globals['_DYNAMICPARAMETERCONSTRAINTS_SINGLECONSTRAINT']._serialized_start=2761
  _globals['_DYNAMICPARAMETERCONSTRAINTS_SINGLECONSTRAINT']._serialized_end=2942
  _globals['_DYNAMICPARAMETERCONSTRAINTS_SINGLECONSTRAINT_EXISTS']._serialized_start=2910
  _globals['_DYNAMICPARAMETERCONSTRAINTS_SINGLECONSTRAINT_EXISTS']._serialized_end=2918
  _globals['_DYNAMICPARAMETERCONSTRAINTS_CONSTRAINTLIST']._serialized_start=2944
  _globals['_DYNAMICPARAMETERCONSTRAINTS_CONSTRAINTLIST']._serialized_end=3038
  _globals['_RESOURCE']._serialized_start=3049
  _globals['_RESOURCE']._serialized_end=3444
  _globals['_RESOURCE_CACHECONTROL']._serialized_start=3378
  _globals['_RESOURCE_CACHECONTROL']._serialized_end=3414
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: envoy/extensions/bootstrap/internal_listener/v3/internal_listener.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'envoy/extensions/bootstrap/internal_listener/v3/internal_listener.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import wrappers_pb2 as google_dot_protobuf_dot_wrappers__pb2
from udpa.annotations import status_pb2 as udpa_dot_annotations_dot_status__pb2
from validate import validate_pb2 as validate_dot_validate__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\nGenvoy/extensions/bootstrap/internal_listener/v3/internal_listener.proto\x12/envoy.extensions.bootstrap.internal_listener.v3\x1a\x1egoogle/protobuf/wrappers.proto\x1a\x1dudpa/annotations/status.proto\x1a\x17validate/validate.proto\"T\n\x10InternalListener\x12@\n\x0e\x62uffer_size_kb\x18\x01 \x01(\x0b\x32\x1c.google.protobuf.UInt32ValueB\n\xfa\x42\x07*\x05\x18\x80@(\x01\x42\xcc\x01\n=io.envoyproxy.envoy.extensions.bootstrap.internal_listener.v3B\x15InternalListenerProtoP\x01Zjgithub.com/envoyproxy/go-control-plane/envoy/extensions/bootstrap/internal_listener/v3;internal_listenerv3\xba\x80\xc8\xd1\x06\x02\x10\x02\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'envoy.extensions.bootstrap.internal_listener.v3.internal_listener_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n=io.envoyproxy.envoy.extensions.bootstrap.internal_listener.v3B\025InternalListenerProtoP\001Zjgithub.com/envoyproxy/go-control-plane/envoy/extensions/bootstrap/internal_listener/v3;internal_listenerv3\272\200\310\321\006\002\020\002'
  _globals['_INTERNALLISTENER'].fields_by_name['buffer_size_kb']._loaded_options = None
  _globals['_INTERNALLISTENER'].fields_by_name['buffer_size_kb']._serialized_options = b'\372B\007*\005\030\200@(\001'
  _globals['_INTERNALLISTENER']._serialized_start=212
  _globals['_INTERNALLISTENER']._serialized_end=296
# @@protoc_insertion_point(module_scope)



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc
import warnings

from envoy.service.ext_proc.v3 import external_processor_pb2 as envoy_dot_service_dot_ext__proc_dot_v3_dot_external__processor__pb2

GRPC_GENERATED_VERSION = '1.74.0'
GRPC_VERSION = grpc.__version__
_version_not_supported = False

try:
    from grpc._utilities import first_version_is_lower
    _version_not_supported = first_version_is_lower(GRPC_VERSION, GRPC_GENERATED_VERSION)
except ImportError:
    _version_not_supported = True

if _version_not_supported:
    raise RuntimeError(
        f'The grpc package installed is at version {GRPC_VERSION},'
        + f' but the generated code in envoy/service/ext_proc/v3/external_processor_pb2_grpc.py depends on'
        + f' grpcio>={GRPC_GENERATED_VERSION}.'
        + f' Please upgrade your grpc module to grpcio>={GRPC_GENERATED_VERSION}'
        + f' or downgrade your generated code using grpcio-tools<={GRPC_VERSION}.'
    )


class ExternalProcessorStub(object):
    """[#protodoc-title: External processing service]

    A service that can access and modify HTTP requests and responses
    as part of a filter chain.
    The overall external processing protocol works like this:

    1. The data plane sends to the service information about the HTTP request.
    2. The service sends back a ProcessingResponse message that directs
    the data plane to either stop processing, continue without it, or send
    it the next chunk of the message body.
    3. If so requested, the data plane sends the server the message body in
    chunks, or the entire body at once. In either case, the server may send
    back a ProcessingResponse for each message it receives, or wait for
    a certain amount of body chunks received before streaming back the
    ProcessingResponse messages.
    4. If so requested, the data plane sends the server the HTTP trailers,
    and the server sends back a ProcessingResponse.
    5. At this point, request processing is done, and we pick up again
    at step 1 when the data plane receives a response from the upstream
    server.
    6. At any point above, if the server closes the gRPC stream cleanly,
    then the data plane proceeds without consulting the server.
    7. At any point above, if the server closes the gRPC stream with an error,
    then the data plane returns a 500 error to the client, unless the filter
    was configured to ignore errors.

    In other words, the process is a request/response conversation, but
    using a gRPC stream to make it easier for the server to
    maintain state.
    """

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.Process = channel.stream_stream(
                '/envoy.service.ext_proc.v3.ExternalProcessor/Process',
                request_serializer=envoy_dot_service_dot_ext__proc_dot_v3_dot_external__processor__pb2.ProcessingRequest.SerializeToString,
                response_deserializer=envoy_dot_service_dot_ext__proc_dot_v3_dot_external__processor__pb2.ProcessingResponse.FromString,
                _registered_method=True)


class ExternalProcessorServicer(object):
    """[#protodoc-title: External processing service]

    A service that can access and modify HTTP requests and responses
    as part of a filter chain.
    The overall external processing protocol works like this:

    1. The data plane sends to the service information about the HTTP request.
    2. The service sends back a ProcessingResponse message that directs
    the data plane to either stop processing, continue without it, or send
    it the next chunk of the message body.
    3. If so requested, the data plane sends the server the message body in
    chunks, or the entire body at once. In either case, the server may send
    back a ProcessingResponse for each message it receives, or wait for
    a certain amount of body chunks received before streaming back the
    ProcessingResponse messages.
    4. If so requested, the data plane sends the server the HTTP trailers,
    and the server sends back a ProcessingResponse.
    5. At this point, request processing is done, and we pick up again
    at step 1 when the data plane receives a response from the upstream
    server.
    6. At any point above, if the server closes the gRPC stream cleanly,
    then the data plane proceeds without consulting the server.
    7. At any point above, if the server closes the gRPC stream with an error,
    then the data plane returns a 500 error to the client, unless the filter
    was configured to ignore errors.

    In other words, the process is a request/response conversation, but
    using a gRPC stream to make it easier for the server to
    maintain state.
    """

    def Process(self, request_iterator, context):
        """This begins the bidirectional stream that the data plane will use to
        give the server control over what the filter does. The actual
        protocol is described by the ProcessingRequest and ProcessingResponse
        messages below.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_ExternalProcessorServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'Process': grpc.stream_stream_rpc_method_handler(
                    servicer.Process,
                    request_deserializer=envoy_dot_service_dot_ext__proc_dot_v3_dot_external__processor__pb2.ProcessingRequest.FromString,
                    response_serializer=envoy_dot_service_dot_ext__proc_dot_v3_dot_external__processor__pb2.ProcessingResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'envoy.service.ext_proc.v3.ExternalProcessor', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))
    server.add_registered_method_handlers('envoy.service.ext_proc.v3.ExternalProcessor', rpc_method_handlers)


 # This class is part of an EXPERIMENTAL API.
class ExternalProcessor(object):
    """[#protodoc-title: External processing service]

    A service that can access and modify HTTP requests and responses
    as part of a filter chain.
    The overall external processing protocol works like this:

    1. The data plane sends to the service information about the HTTP request.
    2. The service sends back a ProcessingResponse message that directs
    the data plane to either stop processing, continue without it, or send
    it the next chunk of the message body.
    3. If so requested, the data plane sends the server the message body in
    chunks, or the entire body at once. In either case, the server may send
    back a ProcessingResponse for each message it receives, or wait for
    a certain amount of body chunks received before streaming back the
    ProcessingResponse messages.
    4. If so requested, the data plane sends the server the HTTP trailers,
    and the server sends back a ProcessingResponse.
    5. At this point, request processing is done, and we pick up again
    at step 1 when the data plane receives a response from the upstream
    server.
    6. At any point above, if the server closes the gRPC stream cleanly,
    then the data plane proceeds without consulting the server.
    7. At any point above, if the server closes the gRPC stream with an error,
    then the data plane returns a 500 error to the client, unless the filter
    was configured to ignore errors.

    In other words, the process is a request/response conversation, but
    using a gRPC stream to make it easier for the server to
    maintain state.
    """

    @staticmethod
    def Process(request_iterator,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.stream_stream(
            request_iterator,
            target,
            '/envoy.service.ext_proc.v3.ExternalProcessor/Process',
            envoy_dot_service_dot_ext__proc_dot_v3_dot_external__processor__pb2.ProcessingRequest.SerializeToString,
            envoy_dot_service_dot_ext__proc_dot_v3_dot_external__processor__pb2.ProcessingResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)



#!/usr/bin/env python3
# Copyright 2026 The gRPC Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Helper script to generate xml file containing http_archive() metadata.

import ast
import base64
import json
import re
import sys
from typing import Any, Dict, List

HttpArchive = Dict[str, Any]
HttpArchives = List[HttpArchive]

CANONICAL_TO_APPARENT_NAME_MAPPING = {
    "@@cel-spec+": "@dev_cel",
    "@@googleapis+": "@com_google_googleapis",
    "@@xds+": "@com_github_cncf_xds",
    "@@protoc-gen-validate+": "@com_envoyproxy_protoc_gen_validate",
    "@@opencensus-proto+": "opencensus_proto",
    "@@envoy_api+": "@envoy_api",
}


# TODO(weizheyuan): Maybe use a mature library for SRI
# parsing so we can support other digest algorithms.
# Supporting only sha256 is fine for now because our
# cmake counterpart download_archive() doesn't support
# other algorithms anyway.
def _integrity_to_sha256(integrity: str) -> str:
    """Convert a SRI to sha256 checksum hex string"""
    matches = re.match("sha256-(.*)", integrity)
    if matches is None:
        return None
    sha256_base64 = matches.group(1)
    sha256_bytes = base64.b64decode(sha256_base64)
    return sha256_bytes.hex()


class KeywordVisitor(ast.NodeVisitor):
    def __init__(self):
        self.http_archive = dict()

    def visit_keyword(self, node):
        if node.arg == "name":
            # Canonical name
            canonical_repo = node.value.value
            apparent_repo = CANONICAL_TO_APPARENT_NAME_MAPPING[
                "@@" + canonical_repo
            ]
            self.http_archive["canonical_repo"] = canonical_repo
            self.http_archive["name"] = apparent_repo[1:]  # strip leading '@'

        elif node.arg == "url":
            self.http_archive["urls"] = [node.value.value]
        elif node.arg == "urls":
            self.http_archive["urls"] = [e.value for e in node.value.elts]
        elif node.arg == "integrity":
            self.http_archive["hash"] = _integrity_to_sha256(node.value.value)
        elif node.arg == "sha256":
            self.http_archive["hash"] = node.value.value
            # consumed by tools/artifact_gen/extract_metadata_from_bazel_xml.cc
            self.http_archive["sha256"] = node.value.value
        elif node.arg == "strip_prefix":
            self.http_archive["strip_prefix"] = node.value.value


class ModuleVisitor(ast.NodeVisitor):
    def __init__(self):
        self.http_archives = list()

    def visit_Call(self, node):
        sub_visitor = KeywordVisitor()
        sub_visitor.visit(node)
        self.http_archives.append(sub_visitor.http_archive)


def parse_http_archives(bazel_output: str) -> HttpArchives:
    module = ast.parse(bazel_output)
    visitor = ModuleVisitor()
    visitor.visit(module)
    return visitor.http_archives


# When invoked as a script, read starlark-like input from `bazel mod show_repo`
# and write json to stdout.
if __name__ == "__main__":
    bazel_output = sys.stdin.read()
    result = parse_http_archives(bazel_output)
    json_obj = {"http_archives": result}
    print(json.dumps(json_obj, indent=4))



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: envoy/extensions/filters/listener/proxy_protocol/v3/proxy_protocol.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'envoy/extensions/filters/listener/proxy_protocol/v3/proxy_protocol.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from envoy.config.core.v3 import proxy_protocol_pb2 as envoy_dot_config_dot_core_dot_v3_dot_proxy__protocol__pb2
from udpa.annotations import status_pb2 as udpa_dot_annotations_dot_status__pb2
from udpa.annotations import versioning_pb2 as udpa_dot_annotations_dot_versioning__pb2
from validate import validate_pb2 as validate_dot_validate__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\nHenvoy/extensions/filters/listener/proxy_protocol/v3/proxy_protocol.proto\x12\x33\x65nvoy.extensions.filters.listener.proxy_protocol.v3\x1a)envoy/config/core/v3/proxy_protocol.proto\x1a\x1dudpa/annotations/status.proto\x1a!udpa/annotations/versioning.proto\x1a\x17validate/validate.proto\"\xfc\x05\n\rProxyProtocol\x12V\n\x05rules\x18\x01 \x03(\x0b\x32G.envoy.extensions.filters.listener.proxy_protocol.v3.ProxyProtocol.Rule\x12-\n%allow_requests_without_proxy_protocol\x18\x02 \x01(\x08\x12M\n\x11pass_through_tlvs\x18\x03 \x01(\x0b\x32\x32.envoy.config.core.v3.ProxyProtocolPassThroughTLVs\x12N\n\x13\x64isallowed_versions\x18\x04 \x03(\x0e\x32\x31.envoy.config.core.v3.ProxyProtocolConfig.Version\x12\x13\n\x0bstat_prefix\x18\x05 \x01(\t\x12\x64\n\x0ctlv_location\x18\x06 \x01(\x0e\x32N.envoy.extensions.filters.listener.proxy_protocol.v3.ProxyProtocol.TlvLocation\x1a@\n\x0cKeyValuePair\x12\x1a\n\x12metadata_namespace\x18\x01 \x01(\t\x12\x14\n\x03key\x18\x02 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01\x1a\x8b\x01\n\x04Rule\x12\x1a\n\x08tlv_type\x18\x01 \x01(\rB\x08\xfa\x42\x05*\x03\x10\x80\x02\x12g\n\x0eon_tlv_present\x18\x02 \x01(\x0b\x32O.envoy.extensions.filters.listener.proxy_protocol.v3.ProxyProtocol.KeyValuePair\"5\n\x0bTlvLocation\x12\x14\n\x10\x44YNAMIC_METADATA\x10\x00\x12\x10\n\x0c\x46ILTER_STATE\x10\x01:C\x9a\xc5\x88\x1e>\n<envoy.config.filter.listener.proxy_protocol.v2.ProxyProtocolB\xce\x01\nAio.envoyproxy.envoy.extensions.filters.listener.proxy_protocol.v3B\x12ProxyProtocolProtoP\x01Zkgithub.com/envoyproxy/go-control-plane/envoy/extensions/filters/listener/proxy_protocol/v3;proxy_protocolv3\xba\x80\xc8\xd1\x06\x02\x10\x02\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'envoy.extensions.filters.listener.proxy_protocol.v3.proxy_protocol_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\nAio.envoyproxy.envoy.extensions.filters.listener.proxy_protocol.v3B\022ProxyProtocolProtoP\001Zkgithub.com/envoyproxy/go-control-plane/envoy/extensions/filters/listener/proxy_protocol/v3;proxy_protocolv3\272\200\310\321\006\002\020\002'
  _globals['_PROXYPROTOCOL_KEYVALUEPAIR'].fields_by_name['key']._loaded_options = None
  _globals['_PROXYPROTOCOL_KEYVALUEPAIR'].fields_by_name['key']._serialized_options = b'\372B\004r\002\020\001'
  _globals['_PROXYPROTOCOL_RULE'].fields_by_name['tlv_type']._loaded_options = None
  _globals['_PROXYPROTOCOL_RULE'].fields_by_name['tlv_type']._serialized_options = b'\372B\005*\003\020\200\002'
  _globals['_PROXYPROTOCOL']._loaded_options = None
  _globals['_PROXYPROTOCOL']._serialized_options = b'\232\305\210\036>\n<envoy.config.filter.listener.proxy_protocol.v2.ProxyProtocol'
  _globals['_PROXYPROTOCOL']._serialized_start=264
  _globals['_PROXYPROTOCOL']._serialized_end=1028
  _globals['_PROXYPROTOCOL_KEYVALUEPAIR']._serialized_start=698
  _globals['_PROXYPROTOCOL_KEYVALUEPAIR']._serialized_end=762
  _globals['_PROXYPROTOCOL_RULE']._serialized_start=765
  _globals['_PROXYPROTOCOL_RULE']._serialized_end=904
  _globals['_PROXYPROTOCOL_TLVLOCATION']._serialized_start=906
  _globals['_PROXYPROTOCOL_TLVLOCATION']._serialized_end=959
# @@protoc_insertion_point(module_scope)



# Copyright 2016 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Test a corner-case at the level of the Cython API."""

import threading
import unittest

from grpc._cython import cygrpc

from tests.unit._cython import test_utilities

_EMPTY_FLAGS = 0
_EMPTY_METADATA = ()


class _ServerDriver:
    def __init__(self, completion_queue, shutdown_tag):
        self._condition = threading.Condition()
        self._completion_queue = completion_queue
        self._shutdown_tag = shutdown_tag
        self._events = []
        self._saw_shutdown_tag = False

    def start(self):
        def in_thread():
            while True:
                event = self._completion_queue.poll()
                with self._condition:
                    self._events.append(event)
                    self._condition.notify()
                    if event.tag is self._shutdown_tag:
                        self._saw_shutdown_tag = True
                        break

        thread = threading.Thread(target=in_thread)
        thread.start()

    def done(self):
        with self._condition:
            return self._saw_shutdown_tag

    def first_event(self):
        with self._condition:
            while not self._events:
                self._condition.wait()
            return self._events[0]

    def events(self):
        with self._condition:
            while not self._saw_shutdown_tag:
                self._condition.wait()
            return tuple(self._events)


class _QueueDriver:
    def __init__(self, condition, completion_queue, due):
        self._condition = condition
        self._completion_queue = completion_queue
        self._due = due
        self._events = []
        self._returned = False

    def start(self):
        def in_thread():
            while True:
                event = self._completion_queue.poll()
                with self._condition:
                    self._events.append(event)
                    self._due.remove(event.tag)
                    self._condition.notify_all()
                    if not self._due:
                        self._returned = True
                        return

        thread = threading.Thread(target=in_thread)
        thread.start()

    def done(self):
        with self._condition:
            return self._returned

    def event_with_tag(self, tag):
        with self._condition:
            while True:
                for event in self._events:
                    if event.tag is tag:
                        return event
                self._condition.wait()

    def events(self):
        with self._condition:
            while not self._returned:
                self._condition.wait()
            return tuple(self._events)


class ReadSomeButNotAllResponsesTest(unittest.TestCase):
    def testReadSomeButNotAllResponses(self):
        server_completion_queue = cygrpc.CompletionQueue()
        server = cygrpc.Server(
            [
                (
                    b"grpc.so_reuseport",
                    0,
                )
            ],
            False,
        )
        server.register_completion_queue(server_completion_queue)
        port = server.add_http2_port(b"[::]:0")
        server.start()
        channel = cygrpc.Channel(
            "localhost:{}".format(port).encode(), set(), None
        )

        server_shutdown_tag = "server_shutdown_tag"
        server_driver = _ServerDriver(
            server_completion_queue, server_shutdown_tag
        )
        server_driver.start()

        client_condition = threading.Condition()
        client_due = set()

        server_call_condition = threading.Condition()
        server_send_initial_metadata_tag = "server_send_initial_metadata_tag"
        server_send_first_message_tag = "server_send_first_message_tag"
        server_send_second_message_tag = "server_send_second_message_tag"
        server_complete_rpc_tag = "server_complete_rpc_tag"
        server_call_due = set(
            (
                server_send_initial_metadata_tag,
                server_send_first_message_tag,
                server_send_second_message_tag,
                server_complete_rpc_tag,
            )
        )
        server_call_completion_queue = cygrpc.CompletionQueue()
        server_call_driver = _QueueDriver(
            server_call_condition, server_call_completion_queue, server_call_due
        )
        server_call_driver.start()

        server_rpc_tag = "server_rpc_tag"
        request_call_result = server.request_call(
            server_call_completion_queue,
            server_completion_queue,
            server_rpc_tag,
        )

        client_receive_initial_metadata_tag = (
            "client_receive_initial_metadata_tag"
        )
        client_complete_rpc_tag = "client_complete_rpc_tag"
        client_call = channel.segregated_call(
            _EMPTY_FLAGS,
            b"/twinkies",
            None,
            None,
            _EMPTY_METADATA,
            None,
            (
                (
                    [
                        cygrpc.ReceiveInitialMetadataOperation(_EMPTY_FLAGS),
                    ],
                    client_receive_initial_metadata_tag,
                ),
                (
                    [
                        cygrpc.SendInitialMetadataOperation(
                            _EMPTY_METADATA, _EMPTY_FLAGS
                        ),
                        cygrpc.SendCloseFromClientOperation(_EMPTY_FLAGS),
                        cygrpc.ReceiveStatusOnClientOperation(_EMPTY_FLAGS),
                    ],
                    client_complete_rpc_tag,
                ),
            ),
        )
        client_receive_initial_metadata_event_future = (
            test_utilities.SimpleFuture(client_call.next_event)
        )

        server_rpc_event = server_driver.first_event()

        with server_call_condition:
            server_send_initial_metadata_start_batch_result = (
                server_rpc_event.call.start_server_batch(
                    [
                        cygrpc.SendInitialMetadataOperation(
                            _EMPTY_METADATA, _EMPTY_FLAGS
                        ),
                    ],
                    server_send_initial_metadata_tag,
                )
            )
            server_send_first_message_start_batch_result = (
                server_rpc_event.call.start_server_batch(
                    [
                        cygrpc.SendMessageOperation(b"\x07", _EMPTY_FLAGS),
                    ],
                    server_send_first_message_tag,
                )
            )
        server_send_initial_metadata_event = server_call_driver.event_with_tag(
            server_send_initial_metadata_tag
        )
        server_send_first_message_event = server_call_driver.event_with_tag(
            server_send_first_message_tag
        )
        with server_call_condition:
            server_send_second_message_start_batch_result = (
                server_rpc_event.call.start_server_batch(
                    [
                        cygrpc.SendMessageOperation(b"\x07", _EMPTY_FLAGS),
                    ],
                    server_send_second_message_tag,
                )
            )
            server_complete_rpc_start_batch_result = (
                server_rpc_event.call.start_server_batch(
                    [
                        cygrpc.ReceiveCloseOnServerOperation(_EMPTY_FLAGS),
                        cygrpc.SendStatusFromServerOperation(
                            (),
                            cygrpc.StatusCode.ok,
                            b"test details",
                            _EMPTY_FLAGS,
                        ),
                    ],
                    server_complete_rpc_tag,
                )
            )
        server_send_second_message_event = server_call_driver.event_with_tag(
            server_send_second_message_tag
        )
        server_complete_rpc_event = server_call_driver.event_with_tag(
            server_complete_rpc_tag
        )
        server_call_driver.events()

        client_receive_initial_metadata_event = (
            client_receive_initial_metadata_event_future.result()
        )

        client_receive_first_message_tag = "client_receive_first_message_tag"
        client_call.operate(
            [
                cygrpc.ReceiveMessageOperation(_EMPTY_FLAGS),
            ],
            client_receive_first_message_tag,
        )
        client_receive_first_message_event = client_call.next_event()

        client_call_cancel_result = client_call.cancel(
            cygrpc.StatusCode.cancelled, "Cancelled during test!"
        )
        client_complete_rpc_event = client_call.next_event()

        channel.close(cygrpc.StatusCode.unknown, "Channel closed!")
        server.shutdown(server_completion_queue, server_shutdown_tag)
        server.cancel_all_calls()
        server_driver.events()

        self.assertEqual(cygrpc.CallError.ok, request_call_result)
        self.assertEqual(
            cygrpc.CallError.ok, server_send_initial_metadata_start_batch_result
        )
        self.assertIs(server_rpc_tag, server_rpc_event.tag)
        self.assertEqual(
            cygrpc.CompletionType.operation_complete,
            server_rpc_event.completion_type,
        )
        self.assertIsInstance(server_rpc_event.call, cygrpc.Call)


if __name__ == "__main__":
    unittest.main(verbosity=2)



# Copyright 2019 The gRPC Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Implementations of interoperability test methods."""

import argparse
import asyncio
import collections
import datetime
import enum
import inspect
import json
import os
import threading
import time
from typing import Any, Optional, Union

from google import auth as google_auth
from google.auth import environment_vars as google_auth_environment_vars
from google.auth.transport import grpc as google_auth_transport_grpc
from google.auth.transport import requests as google_auth_transport_requests
import grpc
from grpc.experimental import aio

from src.proto.grpc.testing import empty_pb2
from src.proto.grpc.testing import messages_pb2
from src.proto.grpc.testing import test_pb2_grpc

_INITIAL_METADATA_KEY = "x-grpc-test-echo-initial"
_TRAILING_METADATA_KEY = "x-grpc-test-echo-trailing-bin"


async def _expect_status_code(
    call: aio.Call, expected_code: grpc.StatusCode
) -> None:
    code = await call.code()
    if code != expected_code:
        raise ValueError(
            "expected code %s, got %s" % (expected_code, await call.code())
        )


async def _expect_status_details(call: aio.Call, expected_details: str) -> None:
    details = await call.details()
    if details != expected_details:
        raise ValueError(
            "expected message %s, got %s"
            % (expected_details, await call.details())
        )


async def _validate_status_code_and_details(
    call: aio.Call, expected_code: grpc.StatusCode, expected_details: str
) -> None:
    await _expect_status_code(call, expected_code)
    await _expect_status_details(call, expected_details)


def _validate_payload_type_and_length(
    response: Union[
        messages_pb2.SimpleResponse, messages_pb2.StreamingOutputCallResponse
    ],
    expected_type: Any,
    expected_length: int,
) -> None:
    if response.payload.type is not expected_type:
        raise ValueError(
            "expected payload type %s, got %s"
            % (expected_type, type(response.payload.type))
        )
    elif len(response.payload.body) != expected_length:
        raise ValueError(
            "expected payload body size %d, got %d"
            % (expected_length, len(response.payload.body))
        )


async def _large_unary_common_behavior(
    stub: test_pb2_grpc.TestServiceStub,
    fill_username: bool,
    fill_oauth_scope: bool,
    call_credentials: Optional[grpc.CallCredentials],
) -> messages_pb2.SimpleResponse:
    size = 314159
    request = messages_pb2.SimpleRequest(
        response_type=messages_pb2.COMPRESSABLE,
        response_size=size,
        payload=messages_pb2.Payload(body=b"\x00" * 271828),
        fill_username=fill_username,
        fill_oauth_scope=fill_oauth_scope,
    )
    response = await stub.UnaryCall(request, credentials=call_credentials)
    _validate_payload_type_and_length(response, messages_pb2.COMPRESSABLE, size)
    return response


async def _empty_unary(stub: test_pb2_grpc.TestServiceStub) -> None:
    response = await stub.EmptyCall(empty_pb2.Empty())
    if not isinstance(response, empty_pb2.Empty):
        raise TypeError(
            'response is of type "%s", not empty_pb2.Empty!' % type(response)
        )


async def _large_unary(stub: test_pb2_grpc.TestServiceStub) -> None:
    await _large_unary_common_behavior(stub, False, False, None)


async def _client_streaming(stub: test_pb2_grpc.TestServiceStub) -> None:
    payload_body_sizes = (
        27182,
        8,
        1828,
        45904,
    )

    async def request_gen():
        for size in payload_body_sizes:
            yield messages_pb2.StreamingInputCallRequest(
                payload=messages_pb2.Payload(body=b"\x00" * size)
            )

    response = await stub.StreamingInputCall(request_gen())
    if response.aggregated_payload_size != sum(payload_body_sizes):
        raise ValueError(
            "incorrect size %d!" % response.aggregated_payload_size
        )


async def _server_streaming(stub: test_pb2_grpc.TestServiceStub) -> None:
    sizes = (
        31415,
        9,
        2653,
        58979,
    )

    request = messages_pb2.StreamingOutputCallRequest(
        response_type=messages_pb2.COMPRESSABLE,
        response_parameters=(
            messages_pb2.ResponseParameters(size=sizes[0]),
            messages_pb2.ResponseParameters(size=sizes[1]),
            messages_pb2.ResponseParameters(size=sizes[2]),
            messages_pb2.ResponseParameters(size=sizes[3]),
        ),
    )
    call = stub.StreamingOutputCall(request)
    for size in sizes:
        response = await call.read()
        _validate_payload_type_and_length(
            response, messages_pb2.COMPRESSABLE, size
        )


async def _ping_pong(stub: test_pb2_grpc.TestServiceStub) -> None:
    request_response_sizes = (
        31415,
        9,
        2653,
        58979,
    )
    request_payload_sizes = (
        27182,
        8,
        1828,
        45904,
    )

    call = stub.FullDuplexCall()
    for response_size, payload_size in zip(
        request_response_sizes, request_payload_sizes
    ):
        request = messages_pb2.StreamingOutputCallRequest(
            response_type=messages_pb2.COMPRESSABLE,
            response_parameters=(
                messages_pb2.ResponseParameters(size=response_size),
            ),
            payload=messages_pb2.Payload(body=b"\x00" * payload_size),
        )

        await call.write(request)
        response = await call.read()
        _validate_payload_type_and_length(
            response, messages_pb2.COMPRESSABLE, response_size
        )
    await call.done_writing()
    await _validate_status_code_and_details(call, grpc.StatusCode.OK, "")


async def _cancel_after_begin(stub: test_pb2_grpc.TestServiceStub):
    call = stub.StreamingInputCall()
    call.cancel()
    if not call.cancelled():
        raise ValueError("expected cancelled method to return True")
    code = await call.code()
    if code is not grpc.StatusCode.CANCELLED:
        raise ValueError("expected status code CANCELLED")


async def _cancel_after_first_response(stub: test_pb2_grpc.TestServiceStub):
    request_response_sizes = (
        31415,
        9,
        2653,
        58979,
    )
    request_payload_sizes = (
        27182,
        8,
        1828,
        45904,
    )

    call = stub.FullDuplexCall()

    response_size = request_response_sizes[0]
    payload_size = request_payload_sizes[0]
    request = messages_pb2.StreamingOutputCallRequest(
        response_type=messages_pb2.COMPRESSABLE,
        response_parameters=(
            messages_pb2.ResponseParameters(size=response_size),
        ),
        payload=messages_pb2.Payload(body=b"\x00" * payload_size),
    )

    await call.write(request)
    await call.read()

    call.cancel()

    try:
        await call.read()
    except asyncio.CancelledError:
        assert await call.code() is grpc.StatusCode.CANCELLED
    else:
        raise ValueError("expected call to be cancelled")


async def _timeout_on_sleeping_server(stub: test_pb2_grpc.TestServiceStub):
    request_payload_size = 27182
    time_limit = datetime.timedelta(seconds=1)

    call = stub.FullDuplexCall(timeout=time_limit.total_seconds())

    request = messages_pb2.StreamingOutputCallRequest(
        response_type=messages_pb2.COMPRESSABLE,
        payload=messages_pb2.Payload(body=b"\x00" * request_payload_size),
        response_parameters=(
            messages_pb2.ResponseParameters(
                interval_us=int(time_limit.total_seconds() * 2 * 10**6)
            ),
        ),
    )
    await call.write(request)
    await call.done_writing()
    try:
        await call.read()
    except aio.AioRpcError as rpc_error:
        if rpc_error.code() is not grpc.StatusCode.DEADLINE_EXCEEDED:
            raise
    else:
        raise ValueError("expected call to exceed deadline")


async def _empty_stream(stub: test_pb2_grpc.TestServiceStub):
    call = stub.FullDuplexCall()
    await call.done_writing()
    assert await call.read() == aio.EOF


async def _status_code_and_message(stub: test_pb2_grpc.TestServiceStub):
    details = "test status message"
    status = grpc.StatusCode.UNKNOWN  # code = 2

    # Test with a UnaryCall
    request = messages_pb2.SimpleRequest(
        response_type=messages_pb2.COMPRESSABLE,
        response_size=1,
        payload=messages_pb2.Payload(body=b"\x00"),
        response_status=messages_pb2.EchoStatus(
            code=status.value[0], message=details
        ),
    )
    call = stub.UnaryCall(request)
    await _validate_status_code_and_details(call, status, details)

    # Test with a FullDuplexCall
    call = stub.FullDuplexCall()
    request = messages_pb2.StreamingOutputCallRequest(
        response_type=messages_pb2.COMPRESSABLE,
        response_parameters=(messages_pb2.ResponseParameters(size=1),),
        payload=messages_pb2.Payload(body=b"\x00"),
        response_status=messages_pb2.EchoStatus(
            code=status.value[0], message=details
        ),
    )
    await call.write(request)  # sends the initial request.
    await call.done_writing()
    try:
        await call.read()
    except aio.AioRpcError as rpc_error:
        assert rpc_error.code() == status
    await _validate_status_code_and_details(call, status, details)


async def _unimplemented_method(stub: test_pb2_grpc.TestServiceStub):
    call = stub.UnimplementedCall(empty_pb2.Empty())
    await _expect_status_code(call, grpc.StatusCode.UNIMPLEMENTED)


async def _unimplemented_service(stub: test_pb2_grpc.UnimplementedServiceStub):
    call = stub.UnimplementedCall(empty_pb2.Empty())
    await _expect_status_code(call, grpc.StatusCode.UNIMPLEMENTED)


async def _custom_metadata(stub: test_pb2_grpc.TestServiceStub):
    initial_metadata_value = "test_initial_metadata_value"
    trailing_metadata_value = b"\x0a\x0b\x0a\x0b\x0a\x0b"
    metadata = aio.Metadata(
        (_INITIAL_METADATA_KEY, initial_metadata_value),
        (_TRAILING_METADATA_KEY, trailing_metadata_value),
    )

    async def _validate_metadata(call):
        initial_metadata = await call.initial_metadata()
        if initial_metadata[_INITIAL_METADATA_KEY] != initial_metadata_value:
            raise ValueError(
                "expected initial metadata %s, got %s"
                % (
                    initial_metadata_value,
                    initial_metadata[_INITIAL_METADATA_KEY],
                )
            )

        trailing_metadata = await call.trailing_metadata()
        if trailing_metadata[_TRAILING_METADATA_KEY] != trailing_metadata_value:
            raise ValueError(
                "expected trailing metadata %s, got %s"
                % (
                    trailing_metadata_value,
                    trailing_metadata[_TRAILING_METADATA_KEY],
                )
            )

    # Testing with UnaryCall
    request = messages_pb2.SimpleRequest(
        response_type=messages_pb2.COMPRESSABLE,
        response_size=1,
        payload=messages_pb2.Payload(body=b"\x00"),
    )
    call = stub.UnaryCall(request, metadata=metadata)
    await _validate_metadata(call)

    # Testing with FullDuplexCall
    call = stub.FullDuplexCall(metadata=metadata)
    request = messages_pb2.StreamingOutputCallRequest(
        response_type=messages_pb2.COMPRESSABLE,
        response_parameters=(messages_pb2.ResponseParameters(size=1),),
    )
    await call.write(request)
    await call.read()
    await call.done_writing()
    await _validate_metadata(call)


async def _compute_engine_creds(
    stub: test_pb2_grpc.TestServiceStub, args: argparse.Namespace
):
    response = await _large_unary_common_behavior(stub, True, True, None)
    if args.default_service_account != response.username:
        raise ValueError(
            "expected username %s, got %s"
            % (args.default_service_account, response.username)
        )


async def _oauth2_auth_token(
    stub: test_pb2_grpc.TestServiceStub, args: argparse.Namespace
):
    json_key_filename = os.environ[google_auth_environment_vars.CREDENTIALS]
    wanted_email = json.load(open(json_key_filename, "r"))["client_email"]
    response = await _large_unary_common_behavior(stub, True, True, None)
    if wanted_email != response.username:
        raise ValueError(
            "expected username %s, got %s" % (wanted_email, response.username)
        )
    if args.oauth_scope.find(response.oauth_scope) == -1:
        raise ValueError(
            'expected to find oauth scope "{}" in received "{}"'.format(
                response.oauth_scope, args.oauth_scope
            )
        )


async def _jwt_token_creds(stub: test_pb2_grpc.TestServiceStub):
    json_key_filename = os.environ[google_auth_environment_vars.CREDENTIALS]
    wanted_email = json.load(open(json_key_filename, "r"))["client_email"]
    response = await _large_unary_common_behavior(stub, True, False, None)
    if wanted_email != response.username:
        raise ValueError(
            "expected username %s, got %s" % (wanted_email, response.username)
        )


async def _per_rpc_creds(
    stub: test_pb2_grpc.TestServiceStub, args: argparse.Namespace
):
    json_key_filename = os.environ[google_auth_environment_vars.CREDENTIALS]
    wanted_email = json.load(open(json_key_filename, "r"))["client_email"]
    google_credentials, unused_project_id = google_auth.default(
        scopes=[args.oauth_scope]
    )
    call_credentials = grpc.metadata_call_credentials(
        google_auth_transport_grpc.AuthMetadataPlugin(
            credentials=google_credentials,
            request=google_auth_transport_requests.Request(),
        )
    )
    response = await _large_unary_common_behavior(
        stub, True, False, call_credentials
    )
    if wanted_email != response.username:
        raise ValueError(
            "expected username %s, got %s" % (wanted_email, response.username)
        )


async def _special_status_message(stub: test_pb2_grpc.TestServiceStub):
    details = (
        b"\t\ntest with whitespace\r\nand Unicode BMP \xe2\x98\xba and non-BMP"
        b" \xf0\x9f\x98\x88\t\n".decode("utf-8")
    )
    status = grpc.StatusCode.UNKNOWN  # code = 2

    # Test with a UnaryCall
    request = messages_pb2.SimpleRequest(
        response_type=messages_pb2.COMPRESSABLE,
        response_size=1,
        payload=messages_pb2.Payload(body=b"\x00"),
        response_status=messages_pb2.EchoStatus(
            code=status.value[0], message=details
        ),
    )
    call = stub.UnaryCall(request)
    await _validate_status_code_and_details(call, status, details)


@enum.unique
class TestCase(enum.Enum):
    EMPTY_UNARY = "empty_unary"
    LARGE_UNARY = "large_unary"
    SERVER_STREAMING = "server_streaming"
    CLIENT_STREAMING = "client_streaming"
    PING_PONG = "ping_pong"
    CANCEL_AFTER_BEGIN = "cancel_after_begin"
    CANCEL_AFTER_FIRST_RESPONSE = "cancel_after_first_response"
    TIMEOUT_ON_SLEEPING_SERVER = "timeout_on_sleeping_server"
    EMPTY_STREAM = "empty_stream"
    STATUS_CODE_AND_MESSAGE = "status_code_and_message"
    UNIMPLEMENTED_METHOD = "unimplemented_method"
    UNIMPLEMENTED_SERVICE = "unimplemented_service"
    CUSTOM_METADATA = "custom_metadata"
    COMPUTE_ENGINE_CREDS = "compute_engine_creds"
    OAUTH2_AUTH_TOKEN = "oauth2_auth_token"
    JWT_TOKEN_CREDS = "jwt_token_creds"
    PER_RPC_CREDS = "per_rpc_creds"
    SPECIAL_STATUS_MESSAGE = "special_status_message"


_TEST_CASE_IMPLEMENTATION_MAPPING = {
    TestCase.EMPTY_UNARY: _empty_unary,
    TestCase.LARGE_UNARY: _large_unary,
    TestCase.SERVER_STREAMING: _server_streaming,
    TestCase.CLIENT_STREAMING: _client_streaming,
    TestCase.PING_PONG: _ping_pong,
    TestCase.CANCEL_AFTER_BEGIN: _cancel_after_begin,
    TestCase.CANCEL_AFTER_FIRST_RESPONSE: _cancel_after_first_response,
    TestCase.TIMEOUT_ON_SLEEPING_SERVER: _timeout_on_sleeping_server,
    TestCase.EMPTY_STREAM: _empty_stream,
    TestCase.STATUS_CODE_AND_MESSAGE: _status_code_and_message,
    TestCase.UNIMPLEMENTED_METHOD: _unimplemented_method,
    TestCase.UNIMPLEMENTED_SERVICE: _unimplemented_service,
    TestCase.CUSTOM_METADATA: _custom_metadata,
    TestCase.COMPUTE_ENGINE_CREDS: _compute_engine_creds,
    TestCase.OAUTH2_AUTH_TOKEN: _oauth2_auth_token,
    TestCase.JWT_TOKEN_CREDS: _jwt_token_creds,
    TestCase.PER_RPC_CREDS: _per_rpc_creds,
    TestCase.SPECIAL_STATUS_MESSAGE: _special_status_message,
}


async def test_interoperability(
    case: TestCase,
    stub: test_pb2_grpc.TestServiceStub,
    args: Optional[argparse.Namespace] = None,
) -> None:
    method = _TEST_CASE_IMPLEMENTATION_MAPPING.get(case)
    if method is None:
        raise NotImplementedError(f'Test case "{case}" not implemented!')
    else:
        num_params = len(inspect.signature(method).parameters)
        if num_params == 1:
            await method(stub)
        elif num_params == 2:
            if args is not None:
                await method(stub, args)
            else:
                raise ValueError(f"Failed to run case [{case}]: args is None")
        else:
            raise ValueError(f"Invalid number of parameters [{num_params}]")



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: envoy/extensions/access_loggers/stats/v3/stats.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'envoy/extensions/access_loggers/stats/v3/stats.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import wrappers_pb2 as google_dot_protobuf_dot_wrappers__pb2
from xds.annotations.v3 import status_pb2 as xds_dot_annotations_dot_v3_dot_status__pb2
from udpa.annotations import status_pb2 as udpa_dot_annotations_dot_status__pb2
from validate import validate_pb2 as validate_dot_validate__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n4envoy/extensions/access_loggers/stats/v3/stats.proto\x12(envoy.extensions.access_loggers.stats.v3\x1a\x1egoogle/protobuf/wrappers.proto\x1a\x1fxds/annotations/v3/status.proto\x1a\x1dudpa/annotations/status.proto\x1a\x17validate/validate.proto\"\xd9\x06\n\x06\x43onfig\x12\x1c\n\x0bstat_prefix\x18\x01 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01\x12N\n\nhistograms\x18\x03 \x03(\x0b\x32:.envoy.extensions.access_loggers.stats.v3.Config.Histogram\x12J\n\x08\x63ounters\x18\x04 \x03(\x0b\x32\x38.envoy.extensions.access_loggers.stats.v3.Config.Counter\x1a;\n\x03Tag\x12\x15\n\x04name\x18\x01 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01\x12\x1d\n\x0cvalue_format\x18\x02 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01\x1a\x61\n\x04Stat\x12\x15\n\x04name\x18\x01 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01\x12\x42\n\x04tags\x18\x02 \x03(\x0b\x32\x34.envoy.extensions.access_loggers.stats.v3.Config.Tag\x1a\xad\x02\n\tHistogram\x12M\n\x04stat\x18\x01 \x01(\x0b\x32\x35.envoy.extensions.access_loggers.stats.v3.Config.StatB\x08\xfa\x42\x05\x8a\x01\x02\x10\x01\x12W\n\x04unit\x18\x02 \x01(\x0e\x32?.envoy.extensions.access_loggers.stats.v3.Config.Histogram.UnitB\x08\xfa\x42\x05\x82\x01\x02\x10\x01\x12#\n\x0cvalue_format\x18\x03 \x01(\tB\r\xfa\x42\nr\x08\x10\x01:\x01%B\x01%\"S\n\x04Unit\x12\x0f\n\x0bUnspecified\x10\x00\x12\t\n\x05\x42ytes\x10\x01\x12\x10\n\x0cMicroseconds\x10\x02\x12\x10\n\x0cMilliseconds\x10\x03\x12\x0b\n\x07Percent\x10\x04\x1a\xba\x01\n\x07\x43ounter\x12M\n\x04stat\x18\x01 \x01(\x0b\x32\x35.envoy.extensions.access_loggers.stats.v3.Config.StatB\x08\xfa\x42\x05\x8a\x01\x02\x10\x01\x12$\n\x0cvalue_format\x18\x02 \x01(\tB\x0e\xfa\x42\x0br\t:\x01%B\x01%\xd0\x01\x01\x12:\n\x0bvalue_fixed\x18\x03 \x01(\x0b\x32\x1c.google.protobuf.UInt64ValueB\x07\xfa\x42\x04\x32\x02 \x00:\x08\xd2\xc6\xa4\xe1\x06\x02\x08\x01\x42\xa7\x01\n6io.envoyproxy.envoy.extensions.access_loggers.stats.v3B\nStatsProtoP\x01ZWgithub.com/envoyproxy/go-control-plane/envoy/extensions/access_loggers/stats/v3;statsv3\xba\x80\xc8\xd1\x06\x02\x10\x02\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'envoy.extensions.access_loggers.stats.v3.stats_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n6io.envoyproxy.envoy.extensions.access_loggers.stats.v3B\nStatsProtoP\001ZWgithub.com/envoyproxy/go-control-plane/envoy/extensions/access_loggers/stats/v3;statsv3\272\200\310\321\006\002\020\002'
  _globals['_CONFIG_TAG'].fields_by_name['name']._loaded_options = None
  _globals['_CONFIG_TAG'].fields_by_name['name']._serialized_options = b'\372B\004r\002\020\001'
  _globals['_CONFIG_TAG'].fields_by_name['value_format']._loaded_options = None
  _globals['_CONFIG_TAG'].fields_by_name['value_format']._serialized_options = b'\372B\004r\002\020\001'
  _globals['_CONFIG_STAT'].fields_by_name['name']._loaded_options = None
  _globals['_CONFIG_STAT'].fields_by_name['name']._serialized_options = b'\372B\004r\002\020\001'
  _globals['_CONFIG_HISTOGRAM'].fields_by_name['stat']._loaded_options = None
  _globals['_CONFIG_HISTOGRAM'].fields_by_name['stat']._serialized_options = b'\372B\005\212\001\002\020\001'
  _globals['_CONFIG_HISTOGRAM'].fields_by_name['unit']._loaded_options = None
  _globals['_CONFIG_HISTOGRAM'].fields_by_name['unit']._serialized_options = b'\372B\005\202\001\002\020\001'
  _globals['_CONFIG_HISTOGRAM'].fields_by_name['value_format']._loaded_options = None
  _globals['_CONFIG_HISTOGRAM'].fields_by_name['value_format']._serialized_options = b'\372B\nr\010\020\001:\001%B\001%'
  _globals['_CONFIG_COUNTER'].fields_by_name['stat']._loaded_options = None
  _globals['_CONFIG_COUNTER'].fields_by_name['stat']._serialized_options = b'\372B\005\212\001\002\020\001'
  _globals['_CONFIG_COUNTER'].fields_by_name['value_format']._loaded_options = None
  _globals['_CONFIG_COUNTER'].fields_by_name['value_format']._serialized_options = b'\372B\013r\t:\001%B\001%\320\001\001'
  _globals['_CONFIG_COUNTER'].fields_by_name['value_fixed']._loaded_options = None
  _globals['_CONFIG_COUNTER'].fields_by_name['value_fixed']._serialized_options = b'\372B\0042\002 \000'
  _globals['_CONFIG'].fields_by_name['stat_prefix']._loaded_options = None
  _globals['_CONFIG'].fields_by_name['stat_prefix']._serialized_options = b'\372B\004r\002\020\001'
  _globals['_CONFIG']._loaded_options = None
  _globals['_CONFIG']._serialized_options = b'\322\306\244\341\006\002\010\001'
  _globals['_CONFIG']._serialized_start=220
  _globals['_CONFIG']._serialized_end=1077
  _globals['_CONFIG_TAG']._serialized_start=416
  _globals['_CONFIG_TAG']._serialized_end=475
  _globals['_CONFIG_STAT']._serialized_start=477
  _globals['_CONFIG_STAT']._serialized_end=574
  _globals['_CONFIG_HISTOGRAM']._serialized_start=577
  _globals['_CONFIG_HISTOGRAM']._serialized_end=878
  _globals['_CONFIG_HISTOGRAM_UNIT']._serialized_start=795
  _globals['_CONFIG_HISTOGRAM_UNIT']._serialized_end=878
  _globals['_CONFIG_COUNTER']._serialized_start=881
  _globals['_CONFIG_COUNTER']._serialized_end=1067
# @@protoc_insertion_point(module_scope)



# Copyright 2019 the gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""The Python implementation of the GRPC helloworld.Greeter client."""

from concurrent import futures
import contextlib
import datetime
import logging
import unittest

# TODO(https://github.com/grpc/grpc/issues/29284)
# isort: off
import grpc
from google.protobuf import duration_pb2
from google.protobuf import timestamp_pb2
from foo.bar import helloworld_pb2
from foo.bar import helloworld_pb2_grpc

# isort: on

_HOST = "localhost"
_SERVER_ADDRESS = "{}:0".format(_HOST)


class Greeter(helloworld_pb2_grpc.GreeterServicer):
    def SayHello(self, request, context):
        request_in_flight = (
            datetime.datetime.now() - request.request_initiation.ToDatetime()
        )
        request_duration = duration_pb2.Duration()
        request_duration.FromTimedelta(request_in_flight)
        return helloworld_pb2.HelloReply(
            message="Hello, %s!" % request.name,
            request_duration=request_duration,
        )


@contextlib.contextmanager
def _listening_server():
    server = grpc.server(futures.ThreadPoolExecutor())
    helloworld_pb2_grpc.add_GreeterServicer_to_server(Greeter(), server)
    port = server.add_insecure_port(_SERVER_ADDRESS)
    server.start()
    try:
        yield port
    finally:
        server.stop(0)


class ImportTest(unittest.TestCase):
    def test_import(self):
        with _listening_server() as port:
            with grpc.insecure_channel("{}:{}".format(_HOST, port)) as channel:
                stub = helloworld_pb2_grpc.GreeterStub(channel)
                request_timestamp = timestamp_pb2.Timestamp()
                request_timestamp.GetCurrentTime()
                response = stub.SayHello(
                    helloworld_pb2.HelloRequest(
                        name="you",
                        request_initiation=request_timestamp,
                    ),
                    wait_for_ready=True,
                )
                self.assertEqual(response.message, "Hello, you!")
                self.assertGreater(response.request_duration.nanos, 0)


if __name__ == "__main__":
    logging.basicConfig()
    unittest.main()



# Copyright 2026 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from cpython.bytes cimport PyBytes_FromStringAndSize
from libcpp.utility cimport move
from cpython cimport PyObject, PyBytes_AsString, PyBytes_GET_SIZE

import threading


cdef StatusOr[string] MakeInternalError(string message):
    return StatusOr[string](Status(AbslStatusCode.kUnknown, message))

cdef StatusOr[string] MakeStringResult(string result):
  return StatusOr[string](result)

cdef class OnCompleteWrapper:
  cdef weak_ptr[PrivateKeySignerPyWrapper.CompletionContext] completion_context

  # Makes this class callable
  def __call__(self, result):
    cdef StatusOr[string] cpp_result
    cdef string cpp_string
    cdef shared_ptr[PrivateKeySignerPyWrapper.CompletionContext] locked_completion_context 
    cdef PrivateKeySignerPyWrapper.CompletionContext* local_completion_context
    if isinstance(result, bytes):
      # We got a signature
      cpp_string = MakeStringForCython(PyBytes_AsString(result), PyBytes_GET_SIZE(result))
      cpp_result = MakeStringResult(cpp_string)
    elif isinstance(result, Exception):
      # If python returns an exception, convert to absl::Status
      cpp_string = MakeStringForCython(PyBytes_AsString(str(result).encode('utf-8')))
      cpp_result = MakeInternalError(cpp_string)
    else:
      # Any other return type is not valid
      cpp_string = MakeStringForCython(PyBytes_AsString(f"Invalid result type: {type(result)}".encode('utf-8')))
      cpp_result = MakeInternalError(cpp_string)
    locked_completion_context = self.completion_context.lock()
    local_completion_context = locked_completion_context.get()
    with nogil:
      if local_completion_context != NULL:
        local_completion_context.OnComplete(cpp_result)

cdef PrivateKeySignerPyWrapper.PrivateKeySignerPyWrapperResult async_sign_wrapper(string_view inp, CSignatureAlgorithm algorithm, PyObject* py_user_sign_fn, weak_ptr[PrivateKeySignerPyWrapper.CompletionContext] completion_context) noexcept nogil:
  cdef string cpp_string
  cdef const char* data
  cdef size_t size
  cdef PrivateKeySignerPyWrapper.PrivateKeySignerPyWrapperResult cpp_result
  with gil:
    # Cast the PyObject* pointer holding the user's python sign impl
    py_user_func = <object>py_user_sign_fn

    py_on_complete_wrapper = OnCompleteWrapper()
    py_on_complete_wrapper.completion_context = move(completion_context)

    # Call the user's Python function and handle results
    py_result = None
    # Mark sync by default, change if the result is async
    cpp_result.is_sync = True
    try:
      data = inp.data()
      size = inp.length()
      py_bytes = PyBytes_FromStringAndSize(data, size)
      py_result = py_user_func(py_bytes, algorithm, py_on_complete_wrapper)
      if isinstance(py_result, bytes):
        # We got a signature
        cpp_string = MakeStringForCython(PyBytes_AsString(py_result), PyBytes_GET_SIZE(py_result))
        cpp_result.sync_result = MakeStringResult(cpp_string)
      elif callable(py_result):
        # Cancellation func
        cpp_result.is_sync = False
        Py_INCREF(py_result)
        cpp_result.async_result.py_user_cancel_fn = <PyObject*> py_result
        cpp_result.async_result.cancel_wrapper = cancel_wrapper
      else:
        # Any other return type is not valid
        cpp_string = MakeStringForCython(PyBytes_AsString(f"Invalid result type: {type(py_result)}".encode('utf-8')))
        cpp_result.sync_result = MakeInternalError(cpp_string)
      return cpp_result
    except Exception as e:
      # If Python raises an exception, make it an error status
      cpp_result.sync_result = MakeInternalError(MakeStringForCython(PyBytes_AsString(f"Exception in user function: {e}".encode('utf-8'))))
      return cpp_result

cdef void cancel_wrapper(PyObject* py_cancel_user_fn) noexcept nogil:
  with gil:
    try:
      py_cancel_func = <object>py_cancel_user_fn
      py_cancel_func()
    except Exception as e:
      # Exceptions in cancellation
      _LOGGER.warning('Exception in user-provided cancellation function: %s', e)


# To be called from the python layer when the user provides a signer function.
cdef shared_ptr[PrivateKeySigner] build_private_key_signer(py_user_func):
  # Run a non-daemon thread that waits for this event to be set. This ensures
  # that the Python interpreter will stay alive long enough for these objects
  # passed into C++ to be properly dereferenced and cleaned up.
  destroy_event = threading.Event()
  destroy_lambda = lambda event=destroy_event: event.wait()
  threading.Thread(target=destroy_lambda, daemon=False).start()
  py_private_key_signer = PrivateKeySignerPyWrapper.Create(async_sign_wrapper, <PyObject*>py_user_func, <PyObject*>destroy_event)
  return py_private_key_signer



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: envoy/config/trace/v2/opencensus.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from envoy.api.v2.core import grpc_service_pb2 as envoy_dot_api_dot_v2_dot_core_dot_grpc__service__pb2
from opencensus.proto.trace.v1 import trace_config_pb2 as opencensus_dot_proto_dot_trace_dot_v1_dot_trace__config__pb2
from udpa.annotations import status_pb2 as udpa_dot_annotations_dot_status__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n&envoy/config/trace/v2/opencensus.proto\x12\x15\x65nvoy.config.trace.v2\x1a$envoy/api/v2/core/grpc_service.proto\x1a,opencensus/proto/trace/v1/trace_config.proto\x1a\x1dudpa/annotations/status.proto\"\xd8\x05\n\x10OpenCensusConfig\x12<\n\x0ctrace_config\x18\x01 \x01(\x0b\x32&.opencensus.proto.trace.v1.TraceConfig\x12\x1f\n\x17stdout_exporter_enabled\x18\x02 \x01(\x08\x12$\n\x1cstackdriver_exporter_enabled\x18\x03 \x01(\x08\x12\x1e\n\x16stackdriver_project_id\x18\x04 \x01(\t\x12\x1b\n\x13stackdriver_address\x18\n \x01(\t\x12@\n\x18stackdriver_grpc_service\x18\r \x01(\x0b\x32\x1e.envoy.api.v2.core.GrpcService\x12\x1f\n\x17zipkin_exporter_enabled\x18\x05 \x01(\x08\x12\x12\n\nzipkin_url\x18\x06 \x01(\t\x12 \n\x18ocagent_exporter_enabled\x18\x0b \x01(\x08\x12\x17\n\x0focagent_address\x18\x0c \x01(\t\x12<\n\x14ocagent_grpc_service\x18\x0e \x01(\x0b\x32\x1e.envoy.api.v2.core.GrpcService\x12T\n\x16incoming_trace_context\x18\x08 \x03(\x0e\x32\x34.envoy.config.trace.v2.OpenCensusConfig.TraceContext\x12T\n\x16outgoing_trace_context\x18\t \x03(\x0e\x32\x34.envoy.config.trace.v2.OpenCensusConfig.TraceContext\"`\n\x0cTraceContext\x12\x08\n\x04NONE\x10\x00\x12\x11\n\rTRACE_CONTEXT\x10\x01\x12\x12\n\x0eGRPC_TRACE_BIN\x10\x02\x12\x17\n\x13\x43LOUD_TRACE_CONTEXT\x10\x03\x12\x06\n\x02\x42\x33\x10\x04J\x04\x08\x07\x10\x08\x42\x86\x01\n#io.envoyproxy.envoy.config.trace.v2B\x0fOpencensusProtoP\x01ZDgithub.com/envoyproxy/go-control-plane/envoy/config/trace/v2;tracev2\xba\x80\xc8\xd1\x06\x02\x10\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'envoy.config.trace.v2.opencensus_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n#io.envoyproxy.envoy.config.trace.v2B\017OpencensusProtoP\001ZDgithub.com/envoyproxy/go-control-plane/envoy/config/trace/v2;tracev2\272\200\310\321\006\002\020\001'
  _globals['_OPENCENSUSCONFIG']._serialized_start=181
  _globals['_OPENCENSUSCONFIG']._serialized_end=909
  _globals['_OPENCENSUSCONFIG_TRACECONTEXT']._serialized_start=807
  _globals['_OPENCENSUSCONFIG_TRACECONTEXT']._serialized_end=903
# @@protoc_insertion_point(module_scope)



# Copyright 2016 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""MetricsService for publishing stress test qps data."""

import time

from src.proto.grpc.testing import metrics_pb2
from src.proto.grpc.testing import metrics_pb2_grpc

GAUGE_NAME = "python_overall_qps"


class MetricsServer(metrics_pb2_grpc.MetricsServiceServicer):
    def __init__(self, histogram):
        self._start_time = time.time()
        self._histogram = histogram

    def _get_qps(self):
        count = self._histogram.get_data().count
        delta = time.time() - self._start_time
        self._histogram.reset()
        self._start_time = time.time()
        return int(count / delta)

    def GetAllGauges(self, request, context):
        qps = self._get_qps()
        return [metrics_pb2.GaugeResponse(name=GAUGE_NAME, long_value=qps)]

    def GetGauge(self, request, context):
        if request.name != GAUGE_NAME:
            raise Exception("Gauge {} does not exist".format(request.name))
        qps = self._get_qps()
        return metrics_pb2.GaugeResponse(name=GAUGE_NAME, long_value=qps)



#!/usr/bin/env python3
# Copyright 2023 The gRPC Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import argparse
import platform
import subprocess
import sys
import time


def test_runner_log(msg):
    sys.stderr.write("\n%s: %s\n" % (__file__, msg))


def python_args(arg_list):
    if platform.system() == "Windows" and arg_list[0].endswith(".py"):
        return [sys.executable] + arg_list
    return arg_list


def wait_until_dns_server_is_up(args):
    for i in range(0, 30):
        test_runner_log(
            "Health check: attempt to connect to DNS server over TCP."
        )
        tcp_connect_subprocess = subprocess.Popen(
            python_args(
                [
                    args.tcp_connect_bin_path,
                    "--server_host",
                    "127.0.0.1",
                    "--server_port",
                    str(args.dns_server_port),
                    "--timeout",
                    str(1),
                ]
            )
        )
        tcp_connect_subprocess.communicate()
        if tcp_connect_subprocess.returncode == 0:
            test_runner_log(
                (
                    "Health check: attempt to make an A-record "
                    "query to DNS server."
                )
            )
            dns_resolver_subprocess = subprocess.Popen(
                python_args(
                    [
                        args.dns_resolver_bin_path,
                        "--qname",
                        "health-check-local-dns-server-is-alive.resolver-tests.grpctestingexp",
                        "--server_host",
                        "127.0.0.1",
                        "--server_port",
                        str(args.dns_server_port),
                    ]
                ),
                stdout=subprocess.PIPE,
            )
            dns_resolver_stdout, _ = dns_resolver_subprocess.communicate(
                str.encode("ascii")
            )
            if dns_resolver_subprocess.returncode == 0:
                if "123.123.123.123".encode("ascii") in dns_resolver_stdout:
                    test_runner_log(
                        (
                            "DNS server is up! "
                            "Successfully reached it over UDP and TCP."
                        )
                    )
                return
        time.sleep(1)
    test_runner_log(
        (
            "Failed to reach DNS server over TCP and/or UDP. "
            "Exiting without running tests."
        )
    )
    sys.exit(1)


def main():
    argp = argparse.ArgumentParser(description="Make DNS queries for A records")
    argp.add_argument(
        "-p",
        "--dns_server_port",
        default=None,
        type=int,
        help=("Port that local DNS server is listening on."),
    )
    argp.add_argument(
        "--dns_resolver_bin_path",
        default=None,
        type=str,
        help=("Path to the DNS health check utility."),
    )
    argp.add_argument(
        "--tcp_connect_bin_path",
        default=None,
        type=str,
        help=("Path to the TCP health check utility."),
    )
    args = argp.parse_args()
    wait_until_dns_server_is_up(args)


if __name__ == "__main__":
    main()



# Copyright 2019 the gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""An example of compression on the server side with gRPC."""

import argparse
from concurrent import futures
import logging
import threading

import grpc
import helloworld_pb2
import helloworld_pb2_grpc

_DESCRIPTION = "A server capable of compression."
_COMPRESSION_OPTIONS = {
    "none": grpc.Compression.NoCompression,
    "deflate": grpc.Compression.Deflate,
    "gzip": grpc.Compression.Gzip,
}
_LOGGER = logging.getLogger(__name__)

_SERVER_HOST = "localhost"


class Greeter(helloworld_pb2_grpc.GreeterServicer):
    def __init__(self, no_compress_every_n):
        super(Greeter, self).__init__()
        self._no_compress_every_n = no_compress_every_n
        self._request_counter = 0
        self._counter_lock = threading.RLock()

    def _should_suppress_compression(self):
        suppress_compression = False
        with self._counter_lock:
            if (
                self._no_compress_every_n
                and self._request_counter % self._no_compress_every_n == 0
            ):
                suppress_compression = True
            self._request_counter += 1
        return suppress_compression

    def SayHello(self, request, context):
        if self._should_suppress_compression():
            context.set_compression(grpc.Compression.NoCompression)
        return helloworld_pb2.HelloReply(message="Hello, %s!" % request.name)


def run_server(server_compression, no_compress_every_n, port):
    server = grpc.server(
        futures.ThreadPoolExecutor(),
        compression=server_compression,
        options=(("grpc.so_reuseport", 1),),
    )
    helloworld_pb2_grpc.add_GreeterServicer_to_server(
        Greeter(no_compress_every_n), server
    )
    address = "{}:{}".format(_SERVER_HOST, port)
    server.add_insecure_port(address)
    server.start()
    print("Server listening at '{}'".format(address))
    server.wait_for_termination()


def main():
    parser = argparse.ArgumentParser(description=_DESCRIPTION)
    parser.add_argument(
        "--server_compression",
        default="none",
        nargs="?",
        choices=_COMPRESSION_OPTIONS.keys(),
        help="The default compression method for the server.",
    )
    parser.add_argument(
        "--no_compress_every_n",
        type=int,
        default=0,
        nargs="?",
        help="If set, every nth reply will be uncompressed.",
    )
    parser.add_argument(
        "--port",
        type=int,
        default=50051,
        nargs="?",
        help="The port on which the server will listen.",
    )
    args = parser.parse_args()
    run_server(
        _COMPRESSION_OPTIONS[args.server_compression],
        args.no_compress_every_n,
        args.port,
    )


if __name__ == "__main__":
    logging.basicConfig()
    main()



# Copyright 2026 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


cdef bint _disable_absl_init_log = os.environ.get("GRPC_PYTHON_DISABLE_ABSL_INIT_LOG", "") in {"1", "t", "true", "y", "yes"}

#
# initialize absl
#
cdef _initialize_absl():
  if _disable_absl_init_log:
    return

  SetStderrThreshold(LogSeverityAtLeast.kInfo)
  InitializeLog()

_initialize_absl()



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: envoy/extensions/matching/common_inputs/transport_socket/v3/transport_socket_inputs.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'envoy/extensions/matching/common_inputs/transport_socket/v3/transport_socket_inputs.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from udpa.annotations import status_pb2 as udpa_dot_annotations_dot_status__pb2
from validate import validate_pb2 as validate_dot_validate__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\nYenvoy/extensions/matching/common_inputs/transport_socket/v3/transport_socket_inputs.proto\x12;envoy.extensions.matching.common_inputs.transport_socket.v3\x1a\x1dudpa/annotations/status.proto\x1a\x17validate/validate.proto\"\xd6\x01\n\x15\x45ndpointMetadataInput\x12\x0e\n\x06\x66ilter\x18\x01 \x01(\t\x12v\n\x04path\x18\x02 \x03(\x0b\x32^.envoy.extensions.matching.common_inputs.transport_socket.v3.EndpointMetadataInput.PathSegmentB\x08\xfa\x42\x05\x92\x01\x02\x08\x01\x1a\x35\n\x0bPathSegment\x12\x16\n\x03key\x18\x01 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01H\x00\x42\x0e\n\x07segment\x12\x03\xf8\x42\x01\"\xd6\x01\n\x15LocalityMetadataInput\x12\x0e\n\x06\x66ilter\x18\x01 \x01(\t\x12v\n\x04path\x18\x02 \x03(\x0b\x32^.envoy.extensions.matching.common_inputs.transport_socket.v3.LocalityMetadataInput.PathSegmentB\x08\xfa\x42\x05\x92\x01\x02\x08\x01\x1a\x35\n\x0bPathSegment\x12\x16\n\x03key\x18\x01 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01H\x00\x42\x0e\n\x07segment\x12\x03\xf8\x42\x01\"(\n\x10\x46ilterStateInput\x12\x14\n\x03key\x18\x01 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01\"2\n\x19TransportSocketNameAction\x12\x15\n\x04name\x18\x01 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01\x42\xe8\x01\nIio.envoyproxy.envoy.extensions.matching.common_inputs.transport_socket.v3B\x1aTransportSocketInputsProtoP\x01Zugithub.com/envoyproxy/go-control-plane/envoy/extensions/matching/common_inputs/transport_socket/v3;transport_socketv3\xba\x80\xc8\xd1\x06\x02\x10\x02\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'envoy.extensions.matching.common_inputs.transport_socket.v3.transport_socket_inputs_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\nIio.envoyproxy.envoy.extensions.matching.common_inputs.transport_socket.v3B\032TransportSocketInputsProtoP\001Zugithub.com/envoyproxy/go-control-plane/envoy/extensions/matching/common_inputs/transport_socket/v3;transport_socketv3\272\200\310\321\006\002\020\002'
  _globals['_ENDPOINTMETADATAINPUT_PATHSEGMENT'].oneofs_by_name['segment']._loaded_options = None
  _globals['_ENDPOINTMETADATAINPUT_PATHSEGMENT'].oneofs_by_name['segment']._serialized_options = b'\370B\001'
  _globals['_ENDPOINTMETADATAINPUT_PATHSEGMENT'].fields_by_name['key']._loaded_options = None
  _globals['_ENDPOINTMETADATAINPUT_PATHSEGMENT'].fields_by_name['key']._serialized_options = b'\372B\004r\002\020\001'
  _globals['_ENDPOINTMETADATAINPUT'].fields_by_name['path']._loaded_options = None
  _globals['_ENDPOINTMETADATAINPUT'].fields_by_name['path']._serialized_options = b'\372B\005\222\001\002\010\001'
  _globals['_LOCALITYMETADATAINPUT_PATHSEGMENT'].oneofs_by_name['segment']._loaded_options = None
  _globals['_LOCALITYMETADATAINPUT_PATHSEGMENT'].oneofs_by_name['segment']._serialized_options = b'\370B\001'
  _globals['_LOCALITYMETADATAINPUT_PATHSEGMENT'].fields_by_name['key']._loaded_options = None
  _globals['_LOCALITYMETADATAINPUT_PATHSEGMENT'].fields_by_name['key']._serialized_options = b'\372B\004r\002\020\001'
  _globals['_LOCALITYMETADATAINPUT'].fields_by_name['path']._loaded_options = None
  _globals['_LOCALITYMETADATAINPUT'].fields_by_name['path']._serialized_options = b'\372B\005\222\001\002\010\001'
  _globals['_FILTERSTATEINPUT'].fields_by_name['key']._loaded_options = None
  _globals['_FILTERSTATEINPUT'].fields_by_name['key']._serialized_options = b'\372B\004r\002\020\001'
  _globals['_TRANSPORTSOCKETNAMEACTION'].fields_by_name['name']._loaded_options = None
  _globals['_TRANSPORTSOCKETNAMEACTION'].fields_by_name['name']._serialized_options = b'\372B\004r\002\020\001'
  _globals['_ENDPOINTMETADATAINPUT']._serialized_start=211
  _globals['_ENDPOINTMETADATAINPUT']._serialized_end=425
  _globals['_ENDPOINTMETADATAINPUT_PATHSEGMENT']._serialized_start=372
  _globals['_ENDPOINTMETADATAINPUT_PATHSEGMENT']._serialized_end=425
  _globals['_LOCALITYMETADATAINPUT']._serialized_start=428
  _globals['_LOCALITYMETADATAINPUT']._serialized_end=642
  _globals['_LOCALITYMETADATAINPUT_PATHSEGMENT']._serialized_start=372
  _globals['_LOCALITYMETADATAINPUT_PATHSEGMENT']._serialized_end=425
  _globals['_FILTERSTATEINPUT']._serialized_start=644
  _globals['_FILTERSTATEINPUT']._serialized_end=684
  _globals['_TRANSPORTSOCKETNAMEACTION']._serialized_start=686
  _globals['_TRANSPORTSOCKETNAMEACTION']._serialized_end=736
# @@protoc_insertion_point(module_scope)



#!/usr/bin/env python3
# Copyright 2017 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Reads stdin to find chttp2_refcount log lines, and prints reference leaks
# to stdout

import collections
import re
import sys


def new_obj():
    return ["destroy"]


outstanding = collections.defaultdict(new_obj)

# Sample log line:
# chttp2:unref:0x629000005200 2->1 destroy [src/core/ext/transport/chttp2/transport/chttp2_transport.c:599]

for line in sys.stdin:
    m = re.search(
        r"chttp2:(  ref|unref):0x([a-fA-F0-9]+) [^ ]+ ([^[]+) \[(.*)\]", line
    )
    if m:
        if m.group(1) == "  ref":
            outstanding[m.group(2)].append(m.group(3))
        else:
            outstanding[m.group(2)].remove(m.group(3))

for obj, remaining in list(outstanding.items()):
    if remaining:
        print(("LEAKED: %s %r" % (obj, remaining)))



# Copyright 2015 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""A setup module for the gRPC Python package."""

import multiprocessing
import os
import sys

# Manually insert the source directory into the Python path for local module
# imports to succeed
sys.path.insert(0, os.path.abspath("."))

import grpc_tools.command
import setuptools

import commands
import grpc_version
import python_version

INSTALL_REQUIRES = (
    "coverage>=7.9.0",
    "grpcio>={version}".format(version=grpc_version.VERSION),
    "grpcio-channelz>={version}".format(version=grpc_version.VERSION),
    "grpcio-status>={version}".format(version=grpc_version.VERSION),
    "grpcio-tools>={version}".format(version=grpc_version.VERSION),
    "grpcio-health-checking>={version}".format(version=grpc_version.VERSION),
    "grpcio-observability>={version}".format(version=grpc_version.VERSION),
    "xds-protos>={version}".format(version=grpc_version.VERSION),
    "oauth2client>=1.4.7",
    "protobuf>=6.33.5,<8.0.0",
    "google-auth>=1.17.2",
    "requests>=2.14.2",
    "absl-py>=1.4.0",
)

COMMAND_CLASS = {
    # Run `preprocess` *before* doing any packaging!
    "preprocess": commands.GatherProto,
    "build_package_protos": commands.BuildPackageProtos,
    "build_py": commands.BuildPy,
    "run_fork": commands.RunFork,
    "run_interop": commands.RunInterop,
    "test_lite": commands.TestLite,
    "test_aio": commands.TestAio,
    "test_py3_only": commands.TestPy3Only,
}

TEST_SUITE = "tests"
TEST_LOADER = "tests:Loader"
TEST_RUNNER = "tests:Runner"
TESTS_REQUIRE = INSTALL_REQUIRES

CLASSIFIERS = [
    "Programming Language :: Python",
    "Programming Language :: Python :: 3",
]

if __name__ == "__main__":
    multiprocessing.freeze_support()
    setuptools.setup(
        install_requires=INSTALL_REQUIRES,
        cmdclass=COMMAND_CLASS,
        classifiers=CLASSIFIERS,
        python_requires=f">={python_version.MIN_PYTHON_VERSION}",
        tests_require=TESTS_REQUIRE,
        test_suite=TEST_SUITE,
        test_loader=TEST_LOADER,
        test_runner=TEST_RUNNER,
    )



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: contrib/envoy/extensions/filters/network/kafka_mesh/v3alpha/kafka_mesh.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'contrib/envoy/extensions/filters/network/kafka_mesh/v3alpha/kafka_mesh.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from xds.annotations.v3 import status_pb2 as xds_dot_annotations_dot_v3_dot_status__pb2
from udpa.annotations import status_pb2 as udpa_dot_annotations_dot_status__pb2
from validate import validate_pb2 as validate_dot_validate__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\nLcontrib/envoy/extensions/filters/network/kafka_mesh/v3alpha/kafka_mesh.proto\x12\x33\x65nvoy.extensions.filters.network.kafka_mesh.v3alpha\x1a\x1fxds/annotations/v3/status.proto\x1a\x1dudpa/annotations/status.proto\x1a\x17validate/validate.proto\"\xb5\x03\n\tKafkaMesh\x12 \n\x0f\x61\x64vertised_host\x18\x01 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01\x12 \n\x0f\x61\x64vertised_port\x18\x02 \x01(\x05\x42\x07\xfa\x42\x04\x1a\x02 \x00\x12\x66\n\x11upstream_clusters\x18\x03 \x03(\x0b\x32K.envoy.extensions.filters.network.kafka_mesh.v3alpha.KafkaClusterDefinition\x12]\n\x10\x66orwarding_rules\x18\x04 \x03(\x0b\x32\x43.envoy.extensions.filters.network.kafka_mesh.v3alpha.ForwardingRule\x12m\n\x13\x63onsumer_proxy_mode\x18\x05 \x01(\x0e\x32P.envoy.extensions.filters.network.kafka_mesh.v3alpha.KafkaMesh.ConsumerProxyMode\".\n\x11\x43onsumerProxyMode\x12\x19\n\x15StatefulConsumerProxy\x10\x00\"\xdf\x03\n\x16KafkaClusterDefinition\x12\x1d\n\x0c\x63luster_name\x18\x01 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01\x12\"\n\x11\x62ootstrap_servers\x18\x02 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01\x12 \n\x0fpartition_count\x18\x03 \x01(\x05\x42\x07\xfa\x42\x04\x1a\x02 \x00\x12x\n\x0fproducer_config\x18\x04 \x03(\x0b\x32_.envoy.extensions.filters.network.kafka_mesh.v3alpha.KafkaClusterDefinition.ProducerConfigEntry\x12x\n\x0f\x63onsumer_config\x18\x05 \x03(\x0b\x32_.envoy.extensions.filters.network.kafka_mesh.v3alpha.KafkaClusterDefinition.ConsumerConfigEntry\x1a\x35\n\x13ProducerConfigEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x1a\x35\n\x13\x43onsumerConfigEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"K\n\x0e\x46orwardingRule\x12\x16\n\x0etarget_cluster\x18\x01 \x01(\t\x12\x16\n\x0ctopic_prefix\x18\x02 \x01(\tH\x00\x42\t\n\x07triggerB\xc9\x01\nAio.envoyproxy.envoy.extensions.filters.network.kafka_mesh.v3alphaB\x0eKafkaMeshProtoP\x01Zbgithub.com/envoyproxy/go-control-plane/contrib/envoy/extensions/filters/network/kafka_mesh/v3alpha\xba\x80\xc8\xd1\x06\x02\x10\x02\xd2\xc6\xa4\xe1\x06\x02\x08\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'contrib.envoy.extensions.filters.network.kafka_mesh.v3alpha.kafka_mesh_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\nAio.envoyproxy.envoy.extensions.filters.network.kafka_mesh.v3alphaB\016KafkaMeshProtoP\001Zbgithub.com/envoyproxy/go-control-plane/contrib/envoy/extensions/filters/network/kafka_mesh/v3alpha\272\200\310\321\006\002\020\002\322\306\244\341\006\002\010\001'
  _globals['_KAFKAMESH'].fields_by_name['advertised_host']._loaded_options = None
  _globals['_KAFKAMESH'].fields_by_name['advertised_host']._serialized_options = b'\372B\004r\002\020\001'
  _globals['_KAFKAMESH'].fields_by_name['advertised_port']._loaded_options = None
  _globals['_KAFKAMESH'].fields_by_name['advertised_port']._serialized_options = b'\372B\004\032\002 \000'
  _globals['_KAFKACLUSTERDEFINITION_PRODUCERCONFIGENTRY']._loaded_options = None
  _globals['_KAFKACLUSTERDEFINITION_PRODUCERCONFIGENTRY']._serialized_options = b'8\001'
  _globals['_KAFKACLUSTERDEFINITION_CONSUMERCONFIGENTRY']._loaded_options = None
  _globals['_KAFKACLUSTERDEFINITION_CONSUMERCONFIGENTRY']._serialized_options = b'8\001'
  _globals['_KAFKACLUSTERDEFINITION'].fields_by_name['cluster_name']._loaded_options = None
  _globals['_KAFKACLUSTERDEFINITION'].fields_by_name['cluster_name']._serialized_options = b'\372B\004r\002\020\001'
  _globals['_KAFKACLUSTERDEFINITION'].fields_by_name['bootstrap_servers']._loaded_options = None
  _globals['_KAFKACLUSTERDEFINITION'].fields_by_name['bootstrap_servers']._serialized_options = b'\372B\004r\002\020\001'
  _globals['_KAFKACLUSTERDEFINITION'].fields_by_name['partition_count']._loaded_options = None
  _globals['_KAFKACLUSTERDEFINITION'].fields_by_name['partition_count']._serialized_options = b'\372B\004\032\002 \000'
  _globals['_KAFKAMESH']._serialized_start=223
  _globals['_KAFKAMESH']._serialized_end=660
  _globals['_KAFKAMESH_CONSUMERPROXYMODE']._serialized_start=614
  _globals['_KAFKAMESH_CONSUMERPROXYMODE']._serialized_end=660
  _globals['_KAFKACLUSTERDEFINITION']._serialized_start=663
  _globals['_KAFKACLUSTERDEFINITION']._serialized_end=1142
  _globals['_KAFKACLUSTERDEFINITION_PRODUCERCONFIGENTRY']._serialized_start=1034
  _globals['_KAFKACLUSTERDEFINITION_PRODUCERCONFIGENTRY']._serialized_end=1087
  _globals['_KAFKACLUSTERDEFINITION_CONSUMERCONFIGENTRY']._serialized_start=1089
  _globals['_KAFKACLUSTERDEFINITION_CONSUMERCONFIGENTRY']._serialized_end=1142
  _globals['_FORWARDINGRULE']._serialized_start=1144
  _globals['_FORWARDINGRULE']._serialized_end=1219
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: envoy/extensions/access_loggers/filters/process_ratelimit/v3/process_ratelimit.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'envoy/extensions/access_loggers/filters/process_ratelimit/v3/process_ratelimit.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from envoy.config.core.v3 import config_source_pb2 as envoy_dot_config_dot_core_dot_v3_dot_config__source__pb2
from udpa.annotations import status_pb2 as udpa_dot_annotations_dot_status__pb2
from validate import validate_pb2 as validate_dot_validate__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\nTenvoy/extensions/access_loggers/filters/process_ratelimit/v3/process_ratelimit.proto\x12<envoy.extensions.access_loggers.filters.process_ratelimit.v3\x1a(envoy/config/core/v3/config_source.proto\x1a\x1dudpa/annotations/status.proto\x1a\x17validate/validate.proto\"\x82\x01\n\x16ProcessRateLimitFilter\x12h\n\x0e\x64ynamic_config\x18\x01 \x01(\x0b\x32P.envoy.extensions.access_loggers.filters.process_ratelimit.v3.DynamicTokenBucket\"y\n\x12\x44ynamicTokenBucket\x12\x1e\n\rresource_name\x18\x01 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01\x12\x43\n\rconfig_source\x18\x02 \x01(\x0b\x32\".envoy.config.core.v3.ConfigSourceB\x08\xfa\x42\x05\x8a\x01\x02\x10\x01\x42\xe6\x01\nJio.envoyproxy.envoy.extensions.access_loggers.filters.process_ratelimit.v3B\x15ProcessRatelimitProtoP\x01Zwgithub.com/envoyproxy/go-control-plane/envoy/extensions/access_loggers/filters/process_ratelimit/v3;process_ratelimitv3\xba\x80\xc8\xd1\x06\x02\x10\x02\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'envoy.extensions.access_loggers.filters.process_ratelimit.v3.process_ratelimit_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\nJio.envoyproxy.envoy.extensions.access_loggers.filters.process_ratelimit.v3B\025ProcessRatelimitProtoP\001Zwgithub.com/envoyproxy/go-control-plane/envoy/extensions/access_loggers/filters/process_ratelimit/v3;process_ratelimitv3\272\200\310\321\006\002\020\002'
  _globals['_DYNAMICTOKENBUCKET'].fields_by_name['resource_name']._loaded_options = None
  _globals['_DYNAMICTOKENBUCKET'].fields_by_name['resource_name']._serialized_options = b'\372B\004r\002\020\001'
  _globals['_DYNAMICTOKENBUCKET'].fields_by_name['config_source']._loaded_options = None
  _globals['_DYNAMICTOKENBUCKET'].fields_by_name['config_source']._serialized_options = b'\372B\005\212\001\002\020\001'
  _globals['_PROCESSRATELIMITFILTER']._serialized_start=249
  _globals['_PROCESSRATELIMITFILTER']._serialized_end=379
  _globals['_DYNAMICTOKENBUCKET']._serialized_start=381
  _globals['_DYNAMICTOKENBUCKET']._serialized_end=502
# @@protoc_insertion_point(module_scope)



# Copyright 2024 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""The Python implementation of the GRPC helloworld.Greeter server with observability enabled."""

from collections import defaultdict
from concurrent import futures
import logging
import time

import grpc
import grpc_observability
import helloworld_pb2
import helloworld_pb2_grpc
import open_telemetry_exporter
from opentelemetry.sdk.metrics import MeterProvider
from opentelemetry.sdk.metrics.export import PeriodicExportingMetricReader

_OTEL_EXPORT_INTERVAL_S = 0.5
_SERVER_PORT = "50051"


class Greeter(helloworld_pb2_grpc.GreeterServicer):
    def SayHello(self, request, context):
        message = request.name
        return helloworld_pb2.HelloReply(message=f"Hello {message}")


def serve():
    all_metrics = defaultdict(list)
    otel_exporter = open_telemetry_exporter.OTelMetricExporter(
        all_metrics, print_live=False
    )
    reader = PeriodicExportingMetricReader(
        exporter=otel_exporter,
        export_interval_millis=_OTEL_EXPORT_INTERVAL_S * 1000,
    )
    provider = MeterProvider(metric_readers=[reader])

    otel_plugin = grpc_observability.OpenTelemetryPlugin(
        meter_provider=provider
    )
    otel_plugin.register_global()

    server = grpc.server(
        thread_pool=futures.ThreadPoolExecutor(max_workers=10),
    )
    helloworld_pb2_grpc.add_GreeterServicer_to_server(Greeter(), server)
    server.add_insecure_port("[::]:" + _SERVER_PORT)
    server.start()
    print("Server started, listening on " + _SERVER_PORT)

    # Sleep to make sure client made RPC call and all metrics are exported.
    time.sleep(10)
    print("Metrics exported on Server side:")
    for metric in all_metrics:
        print(metric)

    server.stop(0)
    otel_plugin.deregister_global()


if __name__ == "__main__":
    logging.basicConfig()
    serve()
